@@ -1,19 +1,13 @@
 import asyncio
 import collections
-import functools
-import sys
-import traceback
 
-from . import helpers
+from .helpers import set_exception, set_result
 from .log import internal_logger
 
-__all__ = (
-    'EofStream', 'StreamReader', 'DataQueue', 'ChunksQueue',
-    'FlowControlStreamReader',
-    'FlowControlDataQueue', 'FlowControlChunksQueue')
 
-PY_35 = sys.version_info >= (3, 5)
-PY_352 = sys.version_info >= (3, 5, 2)
+__all__ = (
+    'EMPTY_PAYLOAD', 'EofStream', 'StreamReader', 'DataQueue',
+    'FlowControlDataQueue')
 
 DEFAULT_LIMIT = 2 ** 16
 
@@ -22,52 +16,60 @@ class EofStream(Exception):
     """eof stream indication."""
 
 
-if PY_35:
-    class AsyncStreamIterator:
+class AsyncStreamIterator:
 
-        def __init__(self, read_func):
-            self.read_func = read_func
+    def __init__(self, read_func):
+        self.read_func = read_func
 
-        def __aiter__(self):
-            return self
+    def __aiter__(self):
+        return self
 
-        if not PY_352:  # pragma: no cover
-            __aiter__ = asyncio.coroutine(__aiter__)
+    async def __anext__(self):
+        try:
+            rv = await self.read_func()
+        except EofStream:
+            raise StopAsyncIteration  # NOQA
+        if rv == b'':
+            raise StopAsyncIteration  # NOQA
+        return rv
 
-        @asyncio.coroutine
-        def __anext__(self):
-            try:
-                rv = yield from self.read_func()
-            except EofStream:
-                raise StopAsyncIteration  # NOQA
-            if rv == b'':
-                raise StopAsyncIteration  # NOQA
-            return rv
+
+class ChunkTupleAsyncStreamIterator(AsyncStreamIterator):
+    async def __anext__(self):
+        rv = await self.read_func()
+        if rv == (b'', False):
+            raise StopAsyncIteration  # NOQA
+        return rv
 
 
 class AsyncStreamReaderMixin:
 
-    if PY_35:
-        def __aiter__(self):
-            return AsyncStreamIterator(self.readline)
+    def __aiter__(self):
+        return AsyncStreamIterator(self.readline)
 
-        if not PY_352:  # pragma: no cover
-            __aiter__ = asyncio.coroutine(__aiter__)
+    def iter_chunked(self, n):
+        """Returns an asynchronous iterator that yields chunks of size n.
 
-        def iter_chunked(self, n):
-            """Returns an asynchronous iterator that yields chunks of size n.
+        Python-3.5 available for Python 3.5+ only
+        """
+        return AsyncStreamIterator(lambda: self.read(n))
 
-            Python-3.5 available for Python 3.5+ only
-            """
-            return AsyncStreamIterator(lambda: self.read(n))
+    def iter_any(self):
+        """Returns an asynchronous iterator that yields all the available
+        data as soon as it is received
 
-        def iter_any(self):
-            """Returns an asynchronous iterator that yields slices of data
-            as they come.
+        Python-3.5 available for Python 3.5+ only
+        """
+        return AsyncStreamIterator(self.readany)
 
-            Python-3.5 available for Python 3.5+ only
-            """
-            return AsyncStreamIterator(self.readany)
+    def iter_chunks(self):
+        """Returns an asynchronous iterator that yields chunks of data
+        as they are received by the server. The yielded objects are tuples
+        of (bytes, bool) as returned by the StreamReader.readchunk method.
+
+        Python-3.5 available for Python 3.5+ only
+        """
+        return ChunkTupleAsyncStreamIterator(self.readchunk)
 
 
 class StreamReader(AsyncStreamReaderMixin):
@@ -86,29 +88,34 @@ class StreamReader(AsyncStreamReaderMixin):
 
     total_bytes = 0
 
-    def __init__(self, limit=DEFAULT_LIMIT, timeout=None, loop=None):
-        self._limit = limit
+    def __init__(self, protocol,
+                 *, limit=DEFAULT_LIMIT, timer=None, loop=None):
+        self._protocol = protocol
+        self._low_water = limit
+        self._high_water = limit * 2
         if loop is None:
             loop = asyncio.get_event_loop()
         self._loop = loop
+        self._size = 0
+        self._cursor = 0
+        self._http_chunk_splits = None
         self._buffer = collections.deque()
-        self._buffer_size = 0
         self._buffer_offset = 0
         self._eof = False
         self._waiter = None
-        self._canceller = None
         self._eof_waiter = None
         self._exception = None
-        self._timeout = timeout
+        self._timer = timer
+        self._eof_callbacks = []
 
     def __repr__(self):
-        info = ['StreamReader']
-        if self._buffer_size:
-            info.append('%d bytes' % self._buffer_size)
+        info = [self.__class__.__name__]
+        if self._size:
+            info.append('%d bytes' % self._size)
         if self._eof:
             info.append('eof')
-        if self._limit != DEFAULT_LIMIT:
-            info.append('l=%d' % self._limit)
+        if self._low_water != DEFAULT_LIMIT:
+            info.append('low=%d high=%d' % (self._low_water, self._high_water))
         if self._waiter:
             info.append('w=%r' % self._waiter)
         if self._exception:
@@ -120,17 +127,26 @@ class StreamReader(AsyncStreamReaderMixin):
 
     def set_exception(self, exc):
         self._exception = exc
+        self._eof_callbacks.clear()
 
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_exception(exc)
+            set_exception(waiter, exc)
 
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
+        waiter = self._eof_waiter
+        if waiter is not None:
+            set_exception(waiter, exc)
+            self._eof_waiter = None
+
+    def on_eof(self, callback):
+        if self._eof:
+            try:
+                callback()
+            except Exception:
+                internal_logger.exception('Exception in eof callback')
+        else:
+            self._eof_callbacks.append(callback)
 
     def feed_eof(self):
         self._eof = True
@@ -138,19 +154,20 @@ class StreamReader(AsyncStreamReaderMixin):
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(True)
-
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
+            set_result(waiter, True)
 
         waiter = self._eof_waiter
         if waiter is not None:
             self._eof_waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(True)
+            set_result(waiter, True)
+
+        for cb in self._eof_callbacks:
+            try:
+                cb()
+            except Exception:
+                internal_logger.exception('Exception in eof callback')
+
+        self._eof_callbacks.clear()
 
     def is_eof(self):
         """Return True if  'feed_eof' was called."""
@@ -160,15 +177,14 @@ class StreamReader(AsyncStreamReaderMixin):
         """Return True if the buffer is empty and 'feed_eof' was called."""
         return self._eof and not self._buffer
 
-    @asyncio.coroutine
-    def wait_eof(self):
+    async def wait_eof(self):
         if self._eof:
             return
 
         assert self._eof_waiter is None
-        self._eof_waiter = helpers.create_future(self._loop)
+        self._eof_waiter = self._loop.create_future()
         try:
-            yield from self._eof_waiter
+            await self._eof_waiter
         finally:
             self._eof_waiter = None
 
@@ -181,32 +197,44 @@ class StreamReader(AsyncStreamReaderMixin):
         if self._buffer_offset:
             self._buffer[0] = self._buffer[0][self._buffer_offset:]
             self._buffer_offset = 0
+        self._size += len(data)
+        self._cursor -= len(data)
         self._buffer.appendleft(data)
-        self._buffer_size += len(data)
+        self._eof_counter = 0
 
-    def feed_data(self, data):
+    # TODO: size is ignored, remove the param later
+    def feed_data(self, data, size=0):
         assert not self._eof, 'feed_data after feed_eof'
 
         if not data:
             return
 
+        self._size += len(data)
         self._buffer.append(data)
-        self._buffer_size += len(data)
         self.total_bytes += len(data)
 
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(False)
+            set_result(waiter, False)
 
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
+        if (self._size > self._high_water and
+                not self._protocol._reading_paused):
+            self._protocol.pause_reading()
 
-    @asyncio.coroutine
-    def _wait(self, func_name):
+    def begin_http_chunk_receiving(self):
+        if self._http_chunk_splits is None:
+            self._http_chunk_splits = []
+
+    def end_http_chunk_receiving(self):
+        if self._http_chunk_splits is None:
+            raise RuntimeError("Called end_chunk_receiving without calling "
+                               "begin_chunk_receiving first")
+        if not self._http_chunk_splits or \
+                self._http_chunk_splits[-1] != self.total_bytes:
+            self._http_chunk_splits.append(self.total_bytes)
+
+    async def _wait(self, func_name):
         # StreamReader uses a future to link the protocol feed_data() method
         # to a read coroutine. Running two read coroutines at the same time
         # would have an unexpected behaviour. It would not possible to know
@@ -214,21 +242,18 @@ class StreamReader(AsyncStreamReaderMixin):
         if self._waiter is not None:
             raise RuntimeError('%s() called while another coroutine is '
                                'already waiting for incoming data' % func_name)
-        waiter = self._waiter = helpers.create_future(self._loop)
-        if self._timeout:
-            self._canceller = self._loop.call_later(self._timeout,
-                                                    self.set_exception,
-                                                    asyncio.TimeoutError())
+
+        waiter = self._waiter = self._loop.create_future()
         try:
-            yield from waiter
+            if self._timer:
+                with self._timer:
+                    await waiter
+            else:
+                await waiter
         finally:
             self._waiter = None
-            if self._canceller is not None:
-                self._canceller.cancel()
-                self._canceller = None
 
-    @asyncio.coroutine
-    def readline(self):
+    async def readline(self):
         if self._exception is not None:
             raise self._exception
 
@@ -247,19 +272,18 @@ class StreamReader(AsyncStreamReaderMixin):
                 if ichar:
                     not_enough = False
 
-                if line_size > self._limit:
+                if line_size > self._high_water:
                     raise ValueError('Line is too long')
 
             if self._eof:
                 break
 
             if not_enough:
-                yield from self._wait('readline')
+                await self._wait('readline')
 
         return b''.join(line)
 
-    @asyncio.coroutine
-    def read(self, n=-1):
+    async def read(self, n=-1):
         if self._exception is not None:
             raise self._exception
 
@@ -271,10 +295,9 @@ class StreamReader(AsyncStreamReaderMixin):
             if self._eof and not self._buffer:
                 self._eof_counter = getattr(self, '_eof_counter', 0) + 1
                 if self._eof_counter > 5:
-                    stack = traceback.format_stack()
                     internal_logger.warning(
                         'Multiple access to StreamReader in eof state, '
-                        'might be infinite loop: \n%s', stack)
+                        'might be infinite loop.', stack_info=True)
 
         if not n:
             return b''
@@ -286,35 +309,63 @@ class StreamReader(AsyncStreamReaderMixin):
             # bytes.  So just call self.readany() until EOF.
             blocks = []
             while True:
-                block = yield from self.readany()
+                block = await self.readany()
                 if not block:
                     break
                 blocks.append(block)
             return b''.join(blocks)
 
         if not self._buffer and not self._eof:
-            yield from self._wait('read')
+            await self._wait('read')
 
         return self._read_nowait(n)
 
-    @asyncio.coroutine
-    def readany(self):
+    async def readany(self):
         if self._exception is not None:
             raise self._exception
 
         if not self._buffer and not self._eof:
-            yield from self._wait('readany')
+            await self._wait('readany')
 
         return self._read_nowait(-1)
 
-    @asyncio.coroutine
-    def readexactly(self, n):
+    async def readchunk(self):
+        """Returns a tuple of (data, end_of_http_chunk). When chunked transfer
+        encoding is used, end_of_http_chunk is a boolean indicating if the end
+        of the data corresponds to the end of a HTTP chunk , otherwise it is
+        always False.
+        """
+        if self._exception is not None:
+            raise self._exception
+
+        if not self._buffer and not self._eof:
+            if (self._http_chunk_splits and
+                    self._cursor == self._http_chunk_splits[0]):
+                # end of http chunk without available data
+                self._http_chunk_splits = self._http_chunk_splits[1:]
+                return (b"", True)
+            await self._wait('readchunk')
+
+        if not self._buffer:
+            # end of file
+            return (b"", False)
+        elif self._http_chunk_splits is not None:
+            while self._http_chunk_splits:
+                pos = self._http_chunk_splits[0]
+                self._http_chunk_splits = self._http_chunk_splits[1:]
+                if pos > self._cursor:
+                    return (self._read_nowait(pos-self._cursor), True)
+            return (self._read_nowait(-1), False)
+        else:
+            return (self._read_nowait_chunk(-1), False)
+
+    async def readexactly(self, n):
         if self._exception is not None:
             raise self._exception
 
         blocks = []
         while n > 0:
-            block = yield from self.read(n)
+            block = await self.read(n)
             if not block:
                 partial = b''.join(blocks)
                 raise asyncio.streams.IncompleteReadError(
@@ -329,7 +380,6 @@ class StreamReader(AsyncStreamReaderMixin):
         #
         # I believe the most users don't know about the method and
         # they are not affected.
-        assert n is not None, "n should be -1"
         if self._exception is not None:
             raise self._exception
 
@@ -354,7 +404,11 @@ class StreamReader(AsyncStreamReaderMixin):
         else:
             data = self._buffer.popleft()
 
-        self._buffer_size -= len(data)
+        self._size -= len(data)
+        self._cursor += len(data)
+
+        if self._size < self._low_water and self._protocol._reading_paused:
+            self._protocol.resume_reading()
         return data
 
     def _read_nowait(self, n):
@@ -379,6 +433,12 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
     def set_exception(self, exc):
         pass
 
+    def on_eof(self, callback):
+        try:
+            callback()
+        except Exception:
+            internal_logger.exception('Exception in eof callback')
+
     def feed_eof(self):
         pass
 
@@ -388,33 +448,34 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
     def at_eof(self):
         return True
 
-    @asyncio.coroutine
-    def wait_eof(self):
+    async def wait_eof(self):
         return
 
     def feed_data(self, data):
         pass
 
-    @asyncio.coroutine
-    def readline(self):
+    async def readline(self):
         return b''
 
-    @asyncio.coroutine
-    def read(self, n=-1):
+    async def read(self, n=-1):
         return b''
 
-    @asyncio.coroutine
-    def readany(self):
+    async def readany(self):
         return b''
 
-    @asyncio.coroutine
-    def readexactly(self, n):
+    async def readchunk(self):
+        return (b'', False)
+
+    async def readexactly(self, n):
         raise asyncio.streams.IncompleteReadError(b'', n)
 
     def read_nowait(self):
         return b''
 
 
+EMPTY_PAYLOAD = EmptyStreamReader()
+
+
 class DataQueue:
     """DataQueue is a general-purpose blocking queue with one reader."""
 
@@ -426,6 +487,9 @@ class DataQueue:
         self._size = 0
         self._buffer = collections.deque()
 
+    def __len__(self):
+        return len(self._buffer)
+
     def is_eof(self):
         return self._eof
 
@@ -436,13 +500,13 @@ class DataQueue:
         return self._exception
 
     def set_exception(self, exc):
+        self._eof = True
         self._exception = exc
 
         waiter = self._waiter
         if waiter is not None:
+            set_exception(waiter, exc)
             self._waiter = None
-            if not waiter.done():
-                waiter.set_exception(exc)
 
     def feed_data(self, data, size=0):
         self._size += size
@@ -451,8 +515,7 @@ class DataQueue:
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(True)
+            set_result(waiter, True)
 
     def feed_eof(self):
         self._eof = True
@@ -460,19 +523,14 @@ class DataQueue:
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(False)
+            set_result(waiter, False)
 
-    @asyncio.coroutine
-    def read(self):
+    async def read(self):
         if not self._buffer and not self._eof:
-            if self._exception is not None:
-                raise self._exception
-
             assert not self._waiter
-            self._waiter = helpers.create_future(self._loop)
+            self._waiter = self._loop.create_future()
             try:
-                yield from self._waiter
+                await self._waiter
             except (asyncio.CancelledError, asyncio.TimeoutError):
                 self._waiter = None
                 raise
@@ -487,118 +545,8 @@ class DataQueue:
             else:
                 raise EofStream
 
-    if PY_35:
-        def __aiter__(self):
-            return AsyncStreamIterator(self.read)
-
-        if not PY_352:  # pragma: no cover
-            __aiter__ = asyncio.coroutine(__aiter__)
-
-
-class ChunksQueue(DataQueue):
-    """Like a :class:`DataQueue`, but for binary chunked data transfer."""
-
-    @asyncio.coroutine
-    def read(self):
-        try:
-            return (yield from super().read())
-        except EofStream:
-            return b''
-
-    readany = read
-
-
-def maybe_resume(func):
-
-    if asyncio.iscoroutinefunction(func):
-        @asyncio.coroutine
-        @functools.wraps(func)
-        def wrapper(self, *args, **kw):
-            result = yield from func(self, *args, **kw)
-            self._check_buffer_size()
-            return result
-    else:
-        @functools.wraps(func)
-        def wrapper(self, *args, **kw):
-            result = func(self, *args, **kw)
-            self._check_buffer_size()
-            return result
-
-    return wrapper
-
-
-class FlowControlStreamReader(StreamReader):
-
-    def __init__(self, stream, limit=DEFAULT_LIMIT, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._stream = stream
-        self._b_limit = limit * 2
-
-        # resume transport reading
-        if stream.paused:
-            try:
-                self._stream.transport.resume_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = False
-
-    def _check_buffer_size(self):
-        if self._stream.paused:
-            if self._buffer_size < self._b_limit:
-                try:
-                    self._stream.transport.resume_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = False
-        else:
-            if self._buffer_size > self._b_limit:
-                try:
-                    self._stream.transport.pause_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = True
-
-    def feed_data(self, data, size=0):
-        has_waiter = self._waiter is not None and not self._waiter.cancelled()
-
-        super().feed_data(data)
-
-        if (not self._stream.paused and
-                not has_waiter and self._buffer_size > self._b_limit):
-            try:
-                self._stream.transport.pause_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = True
-
-    @maybe_resume
-    @asyncio.coroutine
-    def read(self, n=-1):
-        return (yield from super().read(n))
-
-    @maybe_resume
-    @asyncio.coroutine
-    def readline(self):
-        return (yield from super().readline())
-
-    @maybe_resume
-    @asyncio.coroutine
-    def readany(self):
-        return (yield from super().readany())
-
-    @maybe_resume
-    @asyncio.coroutine
-    def readexactly(self, n):
-        return (yield from super().readexactly(n))
-
-    @maybe_resume
-    def read_nowait(self, n=-1):
-        return super().read_nowait(n)
+    def __aiter__(self):
+        return AsyncStreamIterator(self.read)
 
 
 class FlowControlDataQueue(DataQueue):
@@ -606,66 +554,21 @@ class FlowControlDataQueue(DataQueue):
 
     It is a destination for parsed data."""
 
-    def __init__(self, stream, *, limit=DEFAULT_LIMIT, loop=None):
+    def __init__(self, protocol, *, limit=DEFAULT_LIMIT, loop=None):
         super().__init__(loop=loop)
 
-        self._stream = stream
+        self._protocol = protocol
         self._limit = limit * 2
 
-        # resume transport reading
-        if stream.paused:
-            try:
-                self._stream.transport.resume_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = False
-
     def feed_data(self, data, size):
-        has_waiter = self._waiter is not None and not self._waiter.cancelled()
-
         super().feed_data(data, size)
 
-        if (not self._stream.paused and
-                not has_waiter and self._size > self._limit):
-            try:
-                self._stream.transport.pause_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = True
+        if self._size > self._limit and not self._protocol._reading_paused:
+            self._protocol.pause_reading()
 
-    @asyncio.coroutine
-    def read(self):
-        result = yield from super().read()
-
-        if self._stream.paused:
-            if self._size < self._limit:
-                try:
-                    self._stream.transport.resume_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = False
-        else:
-            if self._size > self._limit:
-                try:
-                    self._stream.transport.pause_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = True
-
-        return result
-
-
-class FlowControlChunksQueue(FlowControlDataQueue):
-
-    @asyncio.coroutine
-    def read(self):
+    async def read(self):
         try:
-            return (yield from super().read())
-        except EofStream:
-            return b''
-
-    readany = read
+            return await super().read()
+        finally:
+            if self._size < self._limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
