@@ -1,22 +1,29 @@
 """Async gunicorn worker for aiohttp.web"""
 
 import asyncio
-import logging
 import os
 import re
 import signal
-import socket
-import ssl
 import sys
+from contextlib import suppress
 
-import gunicorn.workers.base as base
 from gunicorn.config import AccessLogFormat as GunicornAccessLogFormat
+from gunicorn.workers import base
 
-from aiohttp.helpers import AccessLogger, ensure_future
-from aiohttp.web_server import Server
-from aiohttp.wsgi import WSGIServerHttpProtocol
+from aiohttp import web
 
-__all__ = ('GunicornWebWorker', 'GunicornUVLoopWebWorker')
+from .helpers import AccessLogger, set_result
+
+
+try:
+    import ssl
+except ImportError:  # pragma: no cover
+    ssl = None
+
+
+__all__ = ('GunicornWebWorker',
+           'GunicornUVLoopWebWorker',
+           'GunicornTokioWebWorker')
 
 
 class GunicornWebWorker(base.Worker):
@@ -27,7 +34,8 @@ class GunicornWebWorker(base.Worker):
     def __init__(self, *args, **kw):  # pragma: no cover
         super().__init__(*args, **kw)
 
-        self.servers = {}
+        self._runner = None
+        self._task = None
         self.exit_code = 0
         self._notify_waiter = None
 
@@ -41,72 +49,33 @@ class GunicornWebWorker(base.Worker):
         super().init_process()
 
     def run(self):
-        if hasattr(self.wsgi, 'startup'):
-            self.loop.run_until_complete(self.wsgi.startup())
-        self._runner = ensure_future(self._run(), loop=self.loop)
+        access_log = self.log.access_log if self.cfg.accesslog else None
+        params = dict(
+            logger=self.log,
+            keepalive_timeout=self.cfg.keepalive,
+            access_log=access_log,
+            access_log_format=self._get_valid_log_format(
+                self.cfg.access_log_format))
+        self._runner = web.AppRunner(self.wsgi, **params)
+        self.loop.run_until_complete(self._runner.setup())
+        self._task = self.loop.create_task(self._run())
 
-        try:
-            self.loop.run_until_complete(self._runner)
-        finally:
-            self.loop.close()
+        with suppress(Exception):  # ignore all finalization problems
+            self.loop.run_until_complete(self._task)
+        if hasattr(self.loop, 'shutdown_asyncgens'):
+            self.loop.run_until_complete(self.loop.shutdown_asyncgens())
+        self.loop.close()
 
         sys.exit(self.exit_code)
 
-    def make_handler(self, app):
-        if hasattr(self.wsgi, 'make_handler'):
-            return app.make_handler(
-                logger=self.log,
-                slow_request_timeout=self.cfg.timeout,
-                keepalive_timeout=self.cfg.keepalive,
-                access_log=self.log.access_log,
-                access_log_format=self._get_valid_log_format(
-                    self.cfg.access_log_format))
-        else:
-            return WSGIServer(self.wsgi, self)
-
-    @asyncio.coroutine
-    def close(self):
-        if self.servers:
-            servers = self.servers
-            self.servers = None
-
-            # stop accepting connections
-            for server, handler in servers.items():
-                self.log.info("Stopping server: %s, connections: %s",
-                              self.pid, len(handler.connections))
-                server.close()
-                yield from server.wait_closed()
-
-            # send on_shutdown event
-            if hasattr(self.wsgi, 'shutdown'):
-                yield from self.wsgi.shutdown()
-
-            # stop alive connections
-            tasks = [
-                handler.shutdown(
-                    timeout=self.cfg.graceful_timeout / 100 * 95)
-                for handler in servers.values()]
-            yield from asyncio.gather(*tasks, loop=self.loop)
-
-            # cleanup application
-            if hasattr(self.wsgi, 'cleanup'):
-                yield from self.wsgi.cleanup()
-
-    @asyncio.coroutine
-    def _run(self):
-
+    async def _run(self):
         ctx = self._create_ssl_context(self.cfg) if self.cfg.is_ssl else None
 
         for sock in self.sockets:
-            handler = self.make_handler(self.wsgi)
-
-            if hasattr(socket, 'AF_UNIX') and sock.family == socket.AF_UNIX:
-                srv = yield from self.loop.create_unix_server(
-                    handler, sock=sock.sock, ssl=ctx)
-            else:
-                srv = yield from self.loop.create_server(
-                    handler, sock=sock.sock, ssl=ctx)
-            self.servers[srv] = handler
+            site = web.SockSite(
+                self._runner, sock, ssl_context=ctx,
+                shutdown_timeout=self.cfg.graceful_timeout / 100 * 95)
+            await site.start()
 
         # If our parent changed then we shut down.
         pid = os.getpid()
@@ -114,8 +83,7 @@ class GunicornWebWorker(base.Worker):
             while self.alive:
                 self.notify()
 
-                cnt = sum(handler.requests_count
-                          for handler in self.servers.values())
+                cnt = self._runner.server.requests_count
                 if self.cfg.max_requests and cnt > self.cfg.max_requests:
                     self.alive = False
                     self.log.info("Max requests, shutting down: %s", self)
@@ -124,27 +92,28 @@ class GunicornWebWorker(base.Worker):
                     self.alive = False
                     self.log.info("Parent changed, shutting down: %s", self)
                 else:
-                    yield from self._wait_next_notify()
-
+                    await self._wait_next_notify()
         except BaseException:
             pass
 
-        yield from self.close()
+        await self._runner.cleanup()
 
     def _wait_next_notify(self):
         self._notify_waiter_done()
 
-        self._notify_waiter = waiter = asyncio.Future(loop=self.loop)
-        self.loop.call_later(1.0, self._notify_waiter_done)
+        self._notify_waiter = waiter = self.loop.create_future()
+        self.loop.call_later(1.0, self._notify_waiter_done, waiter)
 
         return waiter
 
-    def _notify_waiter_done(self):
-        waiter = self._notify_waiter
-        if waiter is not None and not waiter.done():
-            waiter.set_result(True)
+    def _notify_waiter_done(self, waiter=None):
+        if waiter is None:
+            waiter = self._notify_waiter
+        if waiter is not None:
+            set_result(waiter, True)
 
-        self._notify_waiter = None
+        if waiter is self._notify_waiter:
+            self._notify_waiter = None
 
     def init_signals(self):
         # Set up signals through the event loop API.
@@ -178,11 +147,8 @@ class GunicornWebWorker(base.Worker):
         # worker_int callback
         self.cfg.worker_int(self)
 
-        # init closing process
-        self._closing = ensure_future(self.close(), loop=self.loop)
-
-        # close loop
-        self.loop.call_later(0.1, self._notify_waiter_done)
+        # wakeup closing process
+        self._notify_waiter_done()
 
     def handle_abort(self, sig, frame):
         self.alive = False
@@ -196,6 +162,9 @@ class GunicornWebWorker(base.Worker):
 
         See ssl.SSLSocket.__init__ for more details.
         """
+        if ssl is None:  # pragma: no cover
+            raise RuntimeError('SSL is not supported.')
+
         ctx = ssl.SSLContext(cfg.ssl_version)
         ctx.load_cert_chain(cfg.certfile, cfg.keyfile)
         ctx.verify_mode = cfg.cert_reqs
@@ -213,33 +182,13 @@ class GunicornWebWorker(base.Worker):
                 "Gunicorn's style options in form of `%(name)s` are not "
                 "supported for the log formatting. Please use aiohttp's "
                 "format specification to configure access log formatting: "
-                "http://aiohttp.readthedocs.io/en/stable/logging.html"
+                "http://docs.aiohttp.org/en/stable/logging.html"
                 "#format-specification"
             )
         else:
             return source_format
 
 
-class WSGIServer(Server):
-
-    def __init__(self, app, worker):
-        super().__init__(app, loop=worker.loop)
-
-        self.worker = worker
-        self.access_log_format = worker._get_valid_log_format(
-            worker.cfg.access_log_format)
-
-    def __call__(self):
-        return WSGIServerHttpProtocol(
-            self.handler, readpayload=True,
-            loop=self._loop,
-            logger=self.worker.log,
-            debug=self.worker.log.loglevel == logging.DEBUG,
-            keep_alive=self.worker.cfg.keepalive,
-            access_log=self.worker.log.access_log,
-            access_log_format=self.access_log_format)
-
-
 class GunicornUVLoopWebWorker(GunicornWebWorker):
 
     def init_process(self):
@@ -255,3 +204,20 @@ class GunicornUVLoopWebWorker(GunicornWebWorker):
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
 
         super().init_process()
+
+
+class GunicornTokioWebWorker(GunicornWebWorker):
+
+    def init_process(self):  # pragma: no cover
+        import tokio
+
+        # Close any existing event loop before setting a
+        # new policy.
+        asyncio.get_event_loop().close()
+
+        # Setup tokio policy, so that every
+        # asyncio.get_event_loop() will create an instance
+        # of tokio event loop.
+        asyncio.set_event_loop_policy(tokio.EventLoopPolicy())
+
+        super().init_process()
