@@ -1,4 +1,3 @@
-import asyncio
 from unittest import mock
 
 import pytest
@@ -13,50 +12,44 @@ def test_ctor():
     assert view.request is request
 
 
-@asyncio.coroutine
-def test_render_ok():
+async def test_render_ok():
     resp = web.Response(text='OK')
 
     class MyView(View):
-        @asyncio.coroutine
-        def get(self):
+        async def get(self):
             return resp
 
     request = mock.Mock()
     request.method = 'GET'
-    resp2 = yield from MyView(request)
+    resp2 = await MyView(request)
     assert resp is resp2
 
 
-@asyncio.coroutine
-def test_render_unknown_method():
+async def test_render_unknown_method():
 
     class MyView(View):
-        @asyncio.coroutine
-        def get(self):
+        async def get(self):
             return web.Response(text='OK')
         options = get
 
     request = mock.Mock()
     request.method = 'UNKNOWN'
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
-        yield from MyView(request)
+        await MyView(request)
     assert ctx.value.headers['allow'] == 'GET,OPTIONS'
     assert ctx.value.status == 405
 
 
-@asyncio.coroutine
-def test_render_unsupported_method():
+async def test_render_unsupported_method():
 
     class MyView(View):
-        @asyncio.coroutine
-        def get(self):
+        async def get(self):
             return web.Response(text='OK')
         options = delete = get
 
     request = mock.Mock()
     request.method = 'POST'
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
-        yield from MyView(request)
+        await MyView(request)
     assert ctx.value.headers['allow'] == 'DELETE,GET,OPTIONS'
     assert ctx.value.status == 405
