@@ -1,10 +1,12 @@
 # coding: utf-8
 
 import asyncio
+import hashlib
 import io
 import os.path
 import urllib.parse
 import zlib
+from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
@@ -12,12 +14,12 @@ from multidict import CIMultiDict, CIMultiDictProxy, istr
 from yarl import URL
 
 import aiohttp
-from aiohttp import BaseConnector, hdrs, helpers, payload
-from aiohttp.client_reqrep import ClientRequest, ClientResponse
-from aiohttp.helpers import SimpleCookie
+from aiohttp import BaseConnector, hdrs, payload
+from aiohttp.client_reqrep import (ClientRequest, ClientResponse, Fingerprint,
+                                   _merge_ssl_params)
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def make_request(loop):
     request = None
 
@@ -36,39 +38,37 @@ def buf():
     return bytearray()
 
 
-@pytest.yield_fixture
+@pytest.fixture
+def protocol(loop):
+    protocol = mock.Mock()
+    protocol._drain_helper.return_value = loop.create_future()
+    protocol._drain_helper.return_value.set_result(None)
+    return protocol
+
+
+@pytest.fixture
 def transport(buf):
     transport = mock.Mock()
 
     def write(chunk):
         buf.extend(chunk)
 
-    @asyncio.coroutine
-    def write_eof():
+    async def write_eof():
         pass
 
     transport.write.side_effect = write
     transport.write_eof.side_effect = write_eof
+    transport.is_closing.return_value = False
 
     return transport
 
 
 @pytest.fixture
-def conn(stream):
-    return mock.Mock(writer=stream)
-
-
-@pytest.fixture
-def stream(buf, transport):
-    stream = mock.Mock()
-    stream.transport = transport
-
-    def acquire(writer):
-        writer.set_transport(transport)
-
-    stream.acquire.side_effect = acquire
-    stream.drain.return_value = ()
-    return stream
+def conn(transport, protocol):
+    return mock.Mock(
+        transport=transport,
+        protocol=protocol
+    )
 
 
 def test_method1(make_request):
@@ -98,7 +98,9 @@ def test_version_default(make_request):
 
 def test_request_info(make_request):
     req = make_request('get', 'http://python.org/')
-    assert req.request_info == (URL('http://python.org/'), 'GET', req.headers)
+    assert req.request_info == aiohttp.RequestInfo(URL('http://python.org/'),
+                                                   'GET',
+                                                   req.headers)
 
 
 def test_version_err(make_request):
@@ -142,49 +144,49 @@ def test_host_port_default_https(make_request):
     req = make_request('get', 'https://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_nondefault_http(make_request):
     req = make_request('get', 'http://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_nondefault_https(make_request):
     req = make_request('get', 'https://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_default_ws(make_request):
     req = make_request('get', 'ws://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_default_wss(make_request):
     req = make_request('get', 'wss://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_nondefault_ws(make_request):
     req = make_request('get', 'ws://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_nondefault_wss(make_request):
     req = make_request('get', 'wss://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_err(make_request):
@@ -263,7 +265,7 @@ def test_skip_default_useragent_header(make_request):
 
 
 def test_headers(make_request):
-    req = make_request('get', 'http://python.org/',
+    req = make_request('post', 'http://python.org/',
                        headers={'Content-Type': 'text/plain'})
 
     assert 'CONTENT-TYPE' in req.headers
@@ -272,7 +274,7 @@ def test_headers(make_request):
 
 
 def test_headers_list(make_request):
-    req = make_request('get', 'http://python.org/',
+    req = make_request('post', 'http://python.org/',
                        headers=[('Content-Type', 'text/plain')])
     assert 'CONTENT-TYPE' in req.headers
     assert req.headers['CONTENT-TYPE'] == 'text/plain'
@@ -285,15 +287,10 @@ def test_headers_default(make_request):
 
 
 def test_invalid_url(make_request):
-    with pytest.raises(ValueError):
+    with pytest.raises(aiohttp.InvalidURL):
         make_request('get', 'hiwpefhipowhefopw')
 
 
-def test_invalid_idna(make_request):
-    with pytest.raises(ValueError):
-        make_request('get', 'http://\u2061owhefopw.com')
-
-
 def test_no_path(make_request):
     req = make_request('get', 'http://python.org')
     assert '/' == req.url.path
@@ -310,34 +307,33 @@ def test_ipv6_default_https_port(make_request):
     req = make_request('get', 'https://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_ipv6_nondefault_http_port(make_request):
     req = make_request('get', 'http://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_ipv6_nondefault_https_port(make_request):
     req = make_request('get', 'https://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_basic_auth(make_request):
     req = make_request('get', 'http://python.org',
-                       auth=aiohttp.helpers.BasicAuth('nkim', '1234'))
+                       auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
 
 
 def test_basic_auth_utf8(make_request):
     req = make_request('get', 'http://python.org',
-                       auth=aiohttp.helpers.BasicAuth('nkim', 'секрет',
-                                                      'utf-8'))
+                       auth=aiohttp.BasicAuth('nkim', 'секрет', 'utf-8'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbTrRgdC10LrRgNC10YI=' == req.headers['AUTHORIZATION']
 
@@ -505,7 +501,6 @@ def test_gen_netloc_no_port(make_request):
         '012345678901234567890'
 
 
-@asyncio.coroutine
 def test_connection_header(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     req.keep_alive = mock.Mock()
@@ -529,21 +524,19 @@ def test_connection_header(loop, conn):
     assert req.headers.get('CONNECTION') == 'close'
 
 
-@asyncio.coroutine
-def test_no_content_length(loop, conn):
+async def test_no_content_length(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = req.send(conn)
-    assert '0' == req.headers.get('CONTENT-LENGTH')
-    yield from req.close()
+    assert req.headers.get('CONTENT-LENGTH') is None
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_no_content_length2(loop, conn):
+async def test_no_content_length_head(loop, conn):
     req = ClientRequest('head', URL('http://python.org'), loop=loop)
     resp = req.send(conn)
-    assert '0' == req.headers.get('CONTENT-LENGTH')
-    yield from req.close()
+    assert req.headers.get('CONTENT-LENGTH') is None
+    await req.close()
     resp.close()
 
 
@@ -590,7 +583,7 @@ def test_content_type_skip_auto_header_form(loop, conn):
 
 
 def test_content_type_auto_header_content_length_no_skip(loop, conn):
-    req = ClientRequest('get', URL('http://python.org'),
+    req = ClientRequest('post', URL('http://python.org'),
                         data=io.BytesIO(b'hey'),
                         skip_auto_headers={'Content-Length'},
                         loop=loop)
@@ -608,8 +601,7 @@ def test_urlencoded_formdata_charset(loop, conn):
         req.headers.get('CONTENT-TYPE')
 
 
-@asyncio.coroutine
-def test_post_data(loop, conn):
+async def test_post_data(loop, conn):
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -619,23 +611,21 @@ def test_post_data(loop, conn):
         assert b'life=42' == req.body._value
         assert 'application/x-www-form-urlencoded' ==\
             req.headers['CONTENT-TYPE']
-        yield from req.close()
+        await req.close()
         resp.close()
 
 
-@asyncio.coroutine
-def test_pass_falsy_data(loop):
+async def test_pass_falsy_data(loop):
     with mock.patch(
             'aiohttp.client_reqrep.ClientRequest.update_body_from_data'):
         req = ClientRequest(
             'post', URL('http://python.org/'),
             data={}, loop=loop)
         req.update_body_from_data.assert_called_once_with({})
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_pass_falsy_data_file(loop, tmpdir):
+async def test_pass_falsy_data_file(loop, tmpdir):
     testfile = tmpdir.join('tmpfile').open('w+b')
     testfile.write(b'data')
     testfile.seek(0)
@@ -646,22 +636,21 @@ def test_pass_falsy_data_file(loop, tmpdir):
         skip_auto_headers=skip,
         loop=loop)
     assert req.headers.get('CONTENT-LENGTH', None) is not None
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_get_with_data(loop):
+# Elasticsearch API requires to send request body with GET-requests
+async def test_get_with_data(loop):
     for meth in ClientRequest.GET_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'), data={'life': '42'},
             loop=loop)
         assert '/' == req.url.path
         assert b'life=42' == req.body._value
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_bytes_data(loop, conn):
+async def test_bytes_data(loop, conn):
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -671,111 +660,101 @@ def test_bytes_data(loop, conn):
         assert isinstance(req.body, payload.BytesPayload)
         assert b'binary data' == req.body._value
         assert 'application/octet-stream' == req.headers['CONTENT-TYPE']
-        yield from req.close()
+        await req.close()
         resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding(loop, conn):
-    req = ClientRequest('get', URL('http://python.org/'), data='foo',
+async def test_content_encoding(loop, conn):
+    req = ClientRequest('post', URL('http://python.org/'), data='foo',
                         compress='deflate', loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
         resp = req.send(conn)
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
     assert req.headers['CONTENT-ENCODING'] == 'deflate'
     m_writer.return_value\
         .enable_compression.assert_called_with('deflate')
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
-    req = ClientRequest('get', URL('http://python.org/'),
+async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
+    req = ClientRequest('post', URL('http://python.org/'),
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.http'):
         resp = req.send(conn)
     assert 'TRANSFER-ENCODING' not in req.headers
     assert 'CONTENT-ENCODING' not in req.headers
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding_header(loop, conn):
+async def test_content_encoding_header(loop, conn):
     req = ClientRequest(
-        'get', URL('http://python.org/'), data='foo',
+        'post', URL('http://python.org/'), data='foo',
         headers={'Content-Encoding': 'deflate'}, loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
         resp = req.send(conn)
 
     assert not m_writer.return_value.enable_compression.called
     assert not m_writer.return_value.enable_chunking.called
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_compress_and_content_encoding(loop, conn):
+async def test_compress_and_content_encoding(loop, conn):
     with pytest.raises(ValueError):
-        ClientRequest('get', URL('http://python.org/'), data='foo',
+        ClientRequest('post', URL('http://python.org/'), data='foo',
                       headers={'content-encoding': 'deflate'},
                       compress='deflate', loop=loop)
 
 
-@asyncio.coroutine
-def test_chunked(loop, conn):
+async def test_chunked(loop, conn):
     req = ClientRequest(
-        'get', URL('http://python.org/'),
+        'post', URL('http://python.org/'),
         headers={'TRANSFER-ENCODING': 'gzip'}, loop=loop)
     resp = req.send(conn)
     assert 'gzip' == req.headers['TRANSFER-ENCODING']
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked2(loop, conn):
+async def test_chunked2(loop, conn):
     req = ClientRequest(
-        'get', URL('http://python.org/'),
+        'post', URL('http://python.org/'),
         headers={'Transfer-encoding': 'chunked'}, loop=loop)
     resp = req.send(conn)
     assert 'chunked' == req.headers['TRANSFER-ENCODING']
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked_explicit(loop, conn):
+async def test_chunked_explicit(loop, conn):
     req = ClientRequest(
-        'get', URL('http://python.org/'), chunked=True, loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
+        'post', URL('http://python.org/'), chunked=True, loop=loop)
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
         resp = req.send(conn)
 
     assert 'chunked' == req.headers['TRANSFER-ENCODING']
     m_writer.return_value.enable_chunking.assert_called_with()
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked_length(loop, conn):
+async def test_chunked_length(loop, conn):
     with pytest.raises(ValueError):
         ClientRequest(
-            'get', URL('http://python.org/'),
+            'post', URL('http://python.org/'),
             headers={'CONTENT-LENGTH': '1000'}, chunked=True, loop=loop)
 
 
-@asyncio.coroutine
-def test_chunked_transfer_encoding(loop, conn):
+async def test_chunked_transfer_encoding(loop, conn):
     with pytest.raises(ValueError):
         ClientRequest(
-            'get', URL('http://python.org/'),
+            'post', URL('http://python.org/'),
             headers={'TRANSFER-ENCODING': 'chunked'}, chunked=True, loop=loop)
 
 
-@asyncio.coroutine
-def test_file_upload_not_chunked(loop):
+async def test_file_upload_not_chunked(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -785,11 +764,10 @@ def test_file_upload_not_chunked(loop):
             loop=loop)
         assert not req.chunked
         assert req.headers['CONTENT-LENGTH'] == str(os.path.getsize(fname))
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_precompressed_data_stays_intact(loop):
+async def test_precompressed_data_stays_intact(loop):
     data = zlib.compress(b'foobar')
     req = ClientRequest(
         'post', URL('http://python.org/'),
@@ -800,11 +778,10 @@ def test_precompressed_data_stays_intact(loop):
     assert not req.compress
     assert not req.chunked
     assert req.headers['CONTENT-ENCODING'] == 'deflate'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_file_upload_not_chunked_seek(loop):
+async def test_file_upload_not_chunked_seek(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -815,11 +792,10 @@ def test_file_upload_not_chunked_seek(loop):
             loop=loop)
         assert req.headers['CONTENT-LENGTH'] == \
             str(os.path.getsize(fname) - 100)
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_file_upload_force_chunked(loop):
+async def test_file_upload_force_chunked(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -830,7 +806,7 @@ def test_file_upload_force_chunked(loop):
             loop=loop)
         assert req.chunked
         assert 'CONTENT-LENGTH' not in req.headers
-        yield from req.close()
+        await req.close()
 
 
 def test_expect100(loop, conn):
@@ -853,8 +829,7 @@ def test_expect_100_continue_header(loop, conn):
     resp.close()
 
 
-@asyncio.coroutine
-def test_data_stream(loop, buf, conn):
+async def test_data_stream(loop, buf, conn):
     @aiohttp.streamer
     def gen(writer):
         writer.write(b'binary data')
@@ -866,17 +841,15 @@ def test_data_stream(loop, buf, conn):
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
     resp = req.send(conn)
-    assert helpers.isfuture(req._writer)
-    yield from resp.wait_for_close()
+    assert asyncio.isfuture(req._writer)
+    await resp.wait_for_close()
     assert req._writer is None
-
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_file(loop, buf, conn):
+async def test_data_file(loop, buf, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'),
         data=io.BufferedReader(io.BytesIO(b'*' * 2)),
@@ -886,138 +859,130 @@ def test_data_file(loop, buf, conn):
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
     resp = req.send(conn)
-    assert helpers.isfuture(req._writer)
-    yield from resp.wait_for_close()
+    assert asyncio.isfuture(req._writer)
+    await resp.wait_for_close()
     assert req._writer is None
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'2\r\n' + b'*' * 2 + b'\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_exc(loop, conn):
-    fut = helpers.create_future(loop)
+async def test_data_stream_exc(loop, conn):
+    fut = loop.create_future()
 
     @aiohttp.streamer
-    def gen(writer):
+    async def gen(writer):
         writer.write(b'binary data')
-        yield from fut
+        await fut
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
     assert req.chunked
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
-    @asyncio.coroutine
-    def exc():
-        yield from asyncio.sleep(0.01, loop=loop)
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
         fut.set_exception(ValueError)
 
-    helpers.ensure_future(exc(), loop=loop)
+    loop.create_task(throw_exc())
 
     req.send(conn)
-    yield from req._writer
+    await req._writer
     # assert conn.close.called
     assert conn.protocol.set_exception.called
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_exc_chain(loop, conn):
-    fut = helpers.create_future(loop)
+async def test_data_stream_exc_chain(loop, conn):
+    fut = loop.create_future()
 
     @aiohttp.streamer
-    def gen(writer):
-        yield from fut
+    async def gen(writer):
+        await fut
 
     req = ClientRequest('POST', URL('http://python.org/'),
                         data=gen(), loop=loop)
 
     inner_exc = ValueError()
 
-    @asyncio.coroutine
-    def exc():
-        yield from asyncio.sleep(0.01, loop=loop)
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
         fut.set_exception(inner_exc)
 
-    helpers.ensure_future(exc(), loop=loop)
+    loop.create_task(throw_exc())
 
     req.send(conn)
-    yield from req._writer
+    await req._writer
     # assert connection.close.called
     assert conn.protocol.set_exception.called
     outer_exc = conn.protocol.set_exception.call_args[0][0]
     assert isinstance(outer_exc, ValueError)
     assert inner_exc is outer_exc
     assert inner_exc is outer_exc
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_continue(loop, buf, conn):
+async def test_data_stream_continue(loop, buf, conn):
     @aiohttp.streamer
-    def gen(writer):
+    async def gen(writer):
         writer.write(b'binary data')
         writer.write(b' result')
-        yield from writer.write_eof()
+        await writer.write_eof()
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(),
         expect100=True, loop=loop)
     assert req.chunked
 
-    def coro():
-        yield from asyncio.sleep(0.0001, loop=loop)
+    async def coro():
+        await asyncio.sleep(0.0001, loop=loop)
         req._continue.set_result(1)
 
-    helpers.ensure_future(coro(), loop=loop)
+    loop.create_task(coro())
 
     resp = req.send(conn)
-    yield from req._writer
+    await req._writer
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_data_continue(loop, buf, conn):
+async def test_data_continue(loop, buf, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=b'data',
         expect100=True, loop=loop)
 
-    def coro():
-        yield from asyncio.sleep(0.0001, loop=loop)
+    async def coro():
+        await asyncio.sleep(0.0001, loop=loop)
         req._continue.set_result(1)
 
-    helpers.ensure_future(coro(), loop=loop)
+    loop.create_task(coro())
 
     resp = req.send(conn)
 
-    yield from req._writer
+    await req._writer
     assert buf.split(b'\r\n\r\n', 1)[1] == b'data'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_close(loop, buf, conn):
+async def test_close(loop, buf, conn):
     @aiohttp.streamer
-    def gen(writer):
-        yield from asyncio.sleep(0.00001, loop=loop)
+    async def gen(writer):
+        await asyncio.sleep(0.00001, loop=loop)
         writer.write(b'result')
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
     resp = req.send(conn)
-    yield from req.close()
+    await req.close()
     assert buf.split(b'\r\n\r\n', 1)[1] == b'6\r\nresult\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_custom_response_class(loop, conn):
+async def test_custom_response_class(loop, conn):
     class CustomResponse(ClientResponse):
         def read(self, decode=False):
             return 'customized!'
@@ -1027,27 +992,25 @@ def test_custom_response_class(loop, conn):
         loop=loop)
     resp = req.send(conn)
     assert 'customized!' == resp.read()
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_oserror_on_write_bytes(loop, conn):
+async def test_oserror_on_write_bytes(loop, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'), loop=loop)
 
     writer = mock.Mock()
     writer.write.side_effect = OSError
 
-    yield from req.write_bytes(writer, conn)
+    await req.write_bytes(writer, conn)
 
     assert conn.protocol.set_exception.called
     exc = conn.protocol.set_exception.call_args[0][0]
     assert isinstance(exc, aiohttp.ClientOSError)
 
 
-@asyncio.coroutine
-def test_terminate(loop, conn):
+async def test_terminate(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = req.send(conn)
     assert req._writer is not None
@@ -1065,6 +1028,8 @@ def test_terminate_with_closed_loop(loop, conn):
     assert req._writer is not None
     writer = req._writer = mock.Mock()
 
+    loop.run_until_complete(asyncio.sleep(0.05, loop=loop))
+
     loop.close()
     req.terminate()
     assert req._writer is None
@@ -1080,13 +1045,12 @@ def test_terminate_without_writer(loop):
     assert req._writer is None
 
 
-@asyncio.coroutine
-def test_custom_req_rep(loop):
+async def test_custom_req_rep(loop):
     conn = None
 
     class CustomResponse(ClientResponse):
-        @asyncio.coroutine
-        def start(self, connection, read_until_eof=False):
+
+        async def start(self, connection, read_until_eof=False):
             nonlocal conn
             conn = connection
             self.status = 123
@@ -1110,8 +1074,7 @@ def test_custom_req_rep(loop):
             called = True
             return resp
 
-    @asyncio.coroutine
-    def create_connection(req):
+    async def create_connection(req, traces=None):
         assert isinstance(req, CustomRequest)
         return mock.Mock()
     connector = BaseConnector(loop=loop)
@@ -1123,10 +1086,32 @@ def test_custom_req_rep(loop):
         connector=connector,
         loop=loop)
 
-    resp = yield from session.request(
+    resp = await session.request(
         'get', URL('http://example.com/path/to'))
     assert isinstance(resp, CustomResponse)
     assert called
     resp.close()
-    session.close()
+    await session.close()
     conn.close()
+
+
+def test_verify_ssl_false_with_ssl_context(loop):
+    with pytest.warns(DeprecationWarning):
+        with pytest.raises(ValueError):
+            _merge_ssl_params(None, verify_ssl=False,
+                              ssl_context=mock.Mock(), fingerprint=None)
+
+
+def test_bad_fingerprint(loop):
+    with pytest.raises(ValueError):
+        Fingerprint(b'invalid')
+
+
+def test_insecure_fingerprint_md5(loop):
+    with pytest.raises(ValueError):
+        Fingerprint(hashlib.md5(b"foo").digest())
+
+
+def test_insecure_fingerprint_sha1(loop):
+    with pytest.raises(ValueError):
+        Fingerprint(hashlib.sha1(b"foo").digest())
