@@ -1,15 +1,16 @@
 import asyncio
 import datetime
+import itertools
 import os
 import tempfile
 import unittest
+from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
 from yarl import URL
 
-from aiohttp import CookieJar
-from aiohttp.helpers import SimpleCookie
+from aiohttp import CookieJar, DummyCookieJar
 
 
 @pytest.fixture
@@ -577,3 +578,33 @@ class TestCookieJarSafe(TestCookieJarBase):
 
         cookie = cookies_sent["invalid-expires-cookie"]
         self.assertEqual(cookie["expires"], "")
+
+    def test_cookie_not_expired_when_added_after_removal(self):
+        """Test case for https://github.com/aio-libs/aiohttp/issues/2084"""
+        timestamps = [533588.993, 533588.993, 533588.993,
+                      533588.993, 533589.093, 533589.093]
+
+        loop = mock.Mock()
+        loop.time.side_effect = itertools.chain(
+            timestamps, itertools.cycle([timestamps[-1]]))
+
+        jar = CookieJar(unsafe=True, loop=loop)
+        # Remove `foo` cookie.
+        jar.update_cookies(SimpleCookie('foo=""; Max-Age=0'))
+        # Set `foo` cookie to `bar`.
+        jar.update_cookies(SimpleCookie('foo="bar"'))
+
+        # Assert that there is a cookie.
+        assert len(jar) == 1
+
+
+def test_dummy_cookie_jar(loop):
+    cookie = SimpleCookie('foo=bar; Domain=example.com;')
+    dummy_jar = DummyCookieJar(loop=loop)
+    assert len(dummy_jar) == 0
+    dummy_jar.update_cookies(cookie)
+    assert len(dummy_jar) == 0
+    with pytest.raises(StopIteration):
+        next(iter(dummy_jar))
+    assert dummy_jar.filter_cookies(URL("http://example.com/")) is None
+    dummy_jar.clear()
