@@ -15,6 +15,12 @@ from aiohttp.http_parser import (DeflateBuffer, HttpPayloadParser,
                                  HttpRequestParserPy, HttpResponseParserPy)
 
 
+try:
+    import brotli
+except ImportError:
+    brotli = None
+
+
 REQUEST_PARSERS = [HttpRequestParserPy]
 RESPONSE_PARSERS = [HttpResponseParserPy]
 
@@ -66,8 +72,8 @@ test2: data\r
     assert len(messages) == 1
     msg = messages[0][0]
 
-    assert list(msg.headers.items()) == [('Test', 'line continue'),
-                                         ('Test2', 'data')]
+    assert list(msg.headers.items()) == [('test', 'line continue'),
+                                         ('test2', 'data')]
     assert msg.raw_headers == ((b'test', b'line continue'),
                                (b'test2', b'data'))
     assert not msg.should_close
@@ -87,23 +93,21 @@ def test_parse(parser):
     assert msg.version == (1, 1)
 
 
-@asyncio.coroutine
-def test_parse_body(parser):
+async def test_parse_body(parser):
     text = b'GET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
     _, payload = messages[0]
-    body = yield from payload.read(4)
+    body = await payload.read(4)
     assert body == b'body'
 
 
-@asyncio.coroutine
-def test_parse_body_with_CRLF(parser):
+async def test_parse_body_with_CRLF(parser):
     text = b'\r\nGET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
     _, payload = messages[0]
-    body = yield from payload.read(4)
+    body = await payload.read(4)
     assert body == b'body'
 
 
@@ -134,13 +138,34 @@ def test_headers_multi_feed(parser):
     assert len(messages) == 1
 
     msg = messages[0][0]
-    assert list(msg.headers.items()) == [('Test', 'line continue')]
+    assert list(msg.headers.items()) == [('test', 'line continue')]
     assert msg.raw_headers == ((b'test', b'line continue'),)
     assert not msg.should_close
     assert msg.compression is None
     assert not msg.upgrade
 
 
+def test_headers_split_field(parser):
+    text1 = b'GET /test HTTP/1.1\r\n'
+    text2 = b't'
+    text3 = b'es'
+    text4 = b't: value\r\n\r\n'
+
+    messages, upgrade, tail = parser.feed_data(text1)
+    messages, upgrade, tail = parser.feed_data(text2)
+    messages, upgrade, tail = parser.feed_data(text3)
+    assert len(messages) == 0
+    messages, upgrade, tail = parser.feed_data(text4)
+    assert len(messages) == 1
+
+    msg = messages[0][0]
+    assert list(msg.headers.items()) == [('test', 'value')]
+    assert msg.raw_headers == ((b'test', b'value'),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+
+
 def test_parse_headers_multi(parser):
     text = (b'GET /test HTTP/1.1\r\n'
             b'Set-Cookie: c1=cookie1\r\n'
@@ -227,7 +252,7 @@ def test_request_chunked(parser):
     msg, payload = messages[0]
     assert msg.chunked
     assert not upgrade
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
 
 def test_conn_upgrade(parser):
@@ -265,6 +290,15 @@ def test_compression_gzip(parser):
     assert msg.compression == 'gzip'
 
 
+@pytest.mark.skipif(brotli is None, reason="brotli is not installed")
+def test_compression_brotli(parser):
+    text = (b'GET /test HTTP/1.1\r\n'
+            b'content-encoding: br\r\n\r\n')
+    messages, upgrade, tail = parser.feed_data(text)
+    msg = messages[0][0]
+    assert msg.compression == 'br'
+
+
 def test_compression_unknown(parser):
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: compress\r\n\r\n')
@@ -279,7 +313,7 @@ def test_headers_connect(parser):
     messages, upgrade, tail = parser.feed_data(text)
     msg, payload = messages[0]
     assert upgrade
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
 
 def test_headers_old_websocket_key1(parser):
@@ -483,11 +517,12 @@ def test_http_request_chunked_payload(parser):
 
     assert msg.chunked
     assert not payload.is_eof()
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
     parser.feed_data(b'4\r\ndata\r\n4\r\nline\r\n0\r\n\r\n')
 
     assert b'dataline' == b''.join(d for d in payload._buffer)
+    assert [4, 8] == payload._http_chunk_splits
     assert payload.is_eof()
 
 
@@ -502,6 +537,7 @@ def test_http_request_chunked_payload_and_next_message(parser):
         b'transfer-encoding: chunked\r\n\r\n')
 
     assert b'dataline' == b''.join(d for d in payload._buffer)
+    assert [4, 8] == payload._http_chunk_splits
     assert payload.is_eof()
 
     assert len(messages) == 1
@@ -521,14 +557,17 @@ def test_http_request_chunked_payload_chunks(parser):
     parser.feed_data(b'\n4')
     parser.feed_data(b'\r')
     parser.feed_data(b'\n')
-    parser.feed_data(b'line\r\n0\r\n')
+    parser.feed_data(b'li')
+    parser.feed_data(b'ne\r\n0\r\n')
     parser.feed_data(b'test: test\r\n')
 
     assert b'dataline' == b''.join(d for d in payload._buffer)
+    assert [4, 8] == payload._http_chunk_splits
     assert not payload.is_eof()
 
     parser.feed_data(b'\r\n')
     assert b'dataline' == b''.join(d for d in payload._buffer)
+    assert [4, 8] == payload._http_chunk_splits
     assert payload.is_eof()
 
 
@@ -541,6 +580,7 @@ def test_parse_chunked_payload_chunk_extension(parser):
         b'4;test\r\ndata\r\n4\r\nline\r\n0\r\ntest: test\r\n\r\n')
 
     assert b'dataline' == b''.join(d for d in payload._buffer)
+    assert [4, 8] == payload._http_chunk_splits
     assert payload.is_eof()
 
 
@@ -581,6 +621,33 @@ def test_parse_no_length_payload(parser):
     assert payload.is_eof()
 
 
+def test_partial_url(parser):
+    messages, upgrade, tail = parser.feed_data(b'GET /te')
+    assert len(messages) == 0
+    messages, upgrade, tail = parser.feed_data(b'st HTTP/1.1\r\n\r\n')
+    assert len(messages) == 1
+
+    msg, payload = messages[0]
+
+    assert msg.method == 'GET'
+    assert msg.path == '/test'
+    assert msg.version == (1, 1)
+    assert payload.is_eof()
+
+
+def test_url_parse_non_strict_mode(parser):
+    payload = 'GET /test/тест HTTP/1.1\r\n\r\n'.encode('utf-8')
+    messages, upgrade, tail = parser.feed_data(payload)
+    assert len(messages) == 1
+
+    msg, payload = messages[0]
+
+    assert msg.method == 'GET'
+    assert msg.path == '/test/тест'
+    assert msg.version == (1, 1)
+    assert payload.is_eof()
+
+
 class TestParsePayload(unittest.TestCase):
 
     def setUp(self):
@@ -659,6 +726,16 @@ class TestParsePayload(unittest.TestCase):
         self.assertTrue(p.done)
         self.assertTrue(out.is_eof())
 
+    @pytest.mark.skipif(brotli is None, reason="brotli is not installed")
+    def test_http_payload_brotli(self):
+        compressed = brotli.compress(b'brotli data')
+        out = aiohttp.FlowControlDataQueue(self.stream)
+        p = HttpPayloadParser(
+            out, length=len(compressed), compression='br')
+        p.feed_data(compressed)
+        self.assertEqual(b'brotli data', b''.join(d for d, _ in out._buffer))
+        self.assertTrue(out.is_eof())
+
 
 class TestDeflateBuffer(unittest.TestCase):
 
@@ -670,8 +747,8 @@ class TestDeflateBuffer(unittest.TestCase):
         buf = aiohttp.FlowControlDataQueue(self.stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
-        dbuf.zlib = mock.Mock()
-        dbuf.zlib.decompress.return_value = b'line'
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.decompress.return_value = b'line'
 
         dbuf.feed_data(b'data', 4)
         self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
@@ -681,8 +758,8 @@ class TestDeflateBuffer(unittest.TestCase):
         dbuf = DeflateBuffer(buf, 'deflate')
 
         exc = ValueError()
-        dbuf.zlib = mock.Mock()
-        dbuf.zlib.decompress.side_effect = exc
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.decompress.side_effect = exc
 
         self.assertRaises(
             http_exceptions.ContentEncodingError, dbuf.feed_data, b'data', 4)
@@ -691,8 +768,8 @@ class TestDeflateBuffer(unittest.TestCase):
         buf = aiohttp.FlowControlDataQueue(self.stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
-        dbuf.zlib = mock.Mock()
-        dbuf.zlib.flush.return_value = b'line'
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.flush.return_value = b'line'
 
         dbuf.feed_eof()
         self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
@@ -702,9 +779,9 @@ class TestDeflateBuffer(unittest.TestCase):
         buf = aiohttp.FlowControlDataQueue(self.stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
-        dbuf.zlib = mock.Mock()
-        dbuf.zlib.flush.return_value = b'line'
-        dbuf.zlib.eof = False
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.flush.return_value = b'line'
+        dbuf.decompressor.eof = False
 
         self.assertRaises(http_exceptions.ContentEncodingError, dbuf.feed_eof)
 
