@@ -26,20 +26,17 @@ class FakeQueryResult:
         self.host = host
 
 
-@asyncio.coroutine
-def fake_result(addresses):
+async def fake_result(addresses):
     return FakeResult(addresses=tuple(addresses))
 
 
-@asyncio.coroutine
-def fake_query_result(result):
+async def fake_query_result(result):
     return [FakeQueryResult(host=h)
             for h in result]
 
 
 def fake_addrinfo(hosts):
-    @asyncio.coroutine
-    def fake(*args, **kwargs):
+    async def fake(*args, **kwargs):
         if not hosts:
             raise socket.gaierror
 
@@ -50,115 +47,123 @@ def fake_addrinfo(hosts):
 
 
 @pytest.mark.skipif(not gethostbyname, reason="aiodns 1.1 required")
-@asyncio.coroutine
-def test_async_resolver_positive_lookup(loop):
+async def test_async_resolver_positive_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         mock().gethostbyname.return_value = fake_result(['127.0.0.1'])
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.python.org')
+        real = await resolver.resolve('www.python.org')
         ipaddress.ip_address(real[0]['host'])
         mock().gethostbyname.assert_called_with('www.python.org',
                                                 socket.AF_INET)
 
 
 @pytest.mark.skipif(aiodns is None, reason="aiodns required")
-@asyncio.coroutine
-def test_async_resolver_query_positive_lookup(loop):
+async def test_async_resolver_query_positive_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         del mock().gethostbyname
         mock().query.return_value = fake_query_result(['127.0.0.1'])
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.python.org')
+        real = await resolver.resolve('www.python.org')
         ipaddress.ip_address(real[0]['host'])
         mock().query.assert_called_with('www.python.org', 'A')
 
 
 @pytest.mark.skipif(not gethostbyname, reason="aiodns 1.1 required")
-@asyncio.coroutine
-def test_async_resolver_multiple_replies(loop):
+async def test_async_resolver_multiple_replies(loop):
     with patch('aiodns.DNSResolver') as mock:
         ips = ['127.0.0.1', '127.0.0.2', '127.0.0.3', '127.0.0.4']
         mock().gethostbyname.return_value = fake_result(ips)
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.google.com')
+        real = await resolver.resolve('www.google.com')
         ips = [ipaddress.ip_address(x['host']) for x in real]
         assert len(ips) > 3, "Expecting multiple addresses"
 
 
 @pytest.mark.skipif(aiodns is None, reason="aiodns required")
-@asyncio.coroutine
-def test_async_resolver_query_multiple_replies(loop):
+async def test_async_resolver_query_multiple_replies(loop):
     with patch('aiodns.DNSResolver') as mock:
         del mock().gethostbyname
         ips = ['127.0.0.1', '127.0.0.2', '127.0.0.3', '127.0.0.4']
         mock().query.return_value = fake_query_result(ips)
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.google.com')
+        real = await resolver.resolve('www.google.com')
         ips = [ipaddress.ip_address(x['host']) for x in real]
 
 
 @pytest.mark.skipif(not gethostbyname, reason="aiodns 1.1 required")
-@asyncio.coroutine
-def test_async_resolver_negative_lookup(loop):
+async def test_async_resolver_negative_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         mock().gethostbyname.side_effect = aiodns.error.DNSError()
         resolver = AsyncResolver(loop=loop)
-        with pytest.raises(aiodns.error.DNSError):
-            yield from resolver.resolve('doesnotexist.bla')
+        with pytest.raises(OSError):
+            await resolver.resolve('doesnotexist.bla')
 
 
 @pytest.mark.skipif(aiodns is None, reason="aiodns required")
-@asyncio.coroutine
-def test_async_resolver_query_negative_lookup(loop):
+async def test_async_resolver_query_negative_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         del mock().gethostbyname
         mock().query.side_effect = aiodns.error.DNSError()
         resolver = AsyncResolver(loop=loop)
-        with pytest.raises(aiodns.error.DNSError):
-            yield from resolver.resolve('doesnotexist.bla')
+        with pytest.raises(OSError):
+            await resolver.resolve('doesnotexist.bla')
 
 
-@asyncio.coroutine
-def test_threaded_resolver_positive_lookup():
+@pytest.mark.skipif(aiodns is None, reason="aiodns required")
+async def test_async_resolver_no_hosts_in_query(loop):
+    with patch('aiodns.DNSResolver') as mock:
+        del mock().gethostbyname
+        mock().query.return_value = fake_query_result([])
+        resolver = AsyncResolver(loop=loop)
+        with pytest.raises(OSError):
+            await resolver.resolve('doesnotexist.bla')
+
+
+@pytest.mark.skipif(not gethostbyname, reason="aiodns 1.1 required")
+async def test_async_resolver_no_hosts_in_gethostbyname(loop):
+    with patch('aiodns.DNSResolver') as mock:
+        mock().gethostbyname.return_value = fake_result([])
+        resolver = AsyncResolver(loop=loop)
+        with pytest.raises(OSError):
+            await resolver.resolve('doesnotexist.bla')
+
+
+async def test_threaded_resolver_positive_lookup():
     loop = Mock()
     loop.getaddrinfo = fake_addrinfo(["127.0.0.1"])
     resolver = ThreadedResolver(loop=loop)
-    real = yield from resolver.resolve('www.python.org')
+    real = await resolver.resolve('www.python.org')
     ipaddress.ip_address(real[0]['host'])
 
 
-@asyncio.coroutine
-def test_threaded_resolver_multiple_replies():
+async def test_threaded_resolver_multiple_replies():
     loop = Mock()
     ips = ['127.0.0.1', '127.0.0.2', '127.0.0.3', '127.0.0.4']
     loop.getaddrinfo = fake_addrinfo(ips)
     resolver = ThreadedResolver(loop=loop)
-    real = yield from resolver.resolve('www.google.com')
+    real = await resolver.resolve('www.google.com')
     ips = [ipaddress.ip_address(x['host']) for x in real]
     assert len(ips) > 3, "Expecting multiple addresses"
 
 
-@asyncio.coroutine
-def test_threaded_negative_lookup():
+async def test_threaded_negative_lookup():
     loop = Mock()
     ips = []
     loop.getaddrinfo = fake_addrinfo(ips)
     resolver = ThreadedResolver(loop=loop)
     with pytest.raises(socket.gaierror):
-        yield from resolver.resolve('doesnotexist.bla')
+        await resolver.resolve('doesnotexist.bla')
 
 
-@asyncio.coroutine
-def test_close_for_threaded_resolver(loop):
+async def test_close_for_threaded_resolver(loop):
     resolver = ThreadedResolver(loop=loop)
-    yield from resolver.close()
+    await resolver.close()
 
 
 @pytest.mark.skipif(aiodns is None, reason="aiodns required")
-@asyncio.coroutine
-def test_close_for_async_resolver(loop):
+async def test_close_for_async_resolver(loop):
     resolver = AsyncResolver(loop=loop)
-    yield from resolver.close()
+    await resolver.close()
 
 
 def test_default_loop_for_threaded_resolver(loop):
@@ -175,27 +180,25 @@ def test_default_loop_for_async_resolver(loop):
 
 
 @pytest.mark.skipif(not gethostbyname, reason="aiodns 1.1 required")
-@asyncio.coroutine
-def test_async_resolver_ipv6_positive_lookup(loop):
+async def test_async_resolver_ipv6_positive_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         mock().gethostbyname.return_value = fake_result(['::1'])
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.python.org',
-                                           family=socket.AF_INET6)
+        real = await resolver.resolve('www.python.org',
+                                      family=socket.AF_INET6)
         ipaddress.ip_address(real[0]['host'])
         mock().gethostbyname.assert_called_with('www.python.org',
                                                 socket.AF_INET6)
 
 
 @pytest.mark.skipif(aiodns is None, reason="aiodns required")
-@asyncio.coroutine
-def test_async_resolver_query_ipv6_positive_lookup(loop):
+async def test_async_resolver_query_ipv6_positive_lookup(loop):
     with patch('aiodns.DNSResolver') as mock:
         del mock().gethostbyname
         mock().query.return_value = fake_query_result(['::1'])
         resolver = AsyncResolver(loop=loop)
-        real = yield from resolver.resolve('www.python.org',
-                                           family=socket.AF_INET6)
+        real = await resolver.resolve('www.python.org',
+                                      family=socket.AF_INET6)
         ipaddress.ip_address(real[0]['host'])
         mock().query.assert_called_with('www.python.org', 'AAAA')
 
