@@ -1,4 +1,3 @@
-import asyncio
 import os
 import pathlib
 import re
@@ -12,9 +11,9 @@ import aiohttp
 from aiohttp import hdrs, web
 from aiohttp.test_utils import make_mocked_request
 from aiohttp.web import HTTPMethodNotAllowed, HTTPNotFound, Response
-from aiohttp.web_urldispatcher import (AbstractResource, ResourceRoute,
-                                       SystemRoute, UrlDispatcher, View,
-                                       _defaultExpectHandler)
+from aiohttp.web_urldispatcher import (PATH_SEP, AbstractResource,
+                                       ResourceRoute, SystemRoute, View,
+                                       _default_expect_handler)
 
 
 def make_request(method, path):
@@ -23,8 +22,7 @@ def make_request(method, path):
 
 def make_handler():
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         return Response(request)  # pragma: no cover
 
     return handler
@@ -32,7 +30,9 @@ def make_handler():
 
 @pytest.fixture
 def app(loop):
-    return web.Application(loop=loop)
+    app = web.Application()
+    app._set_loop(loop)
+    return app
 
 
 @pytest.fixture
@@ -69,131 +69,131 @@ def test_register_uncommon_http_methods(router):
         router.add_route(method, '/handler/to/path', make_handler())
 
 
-@asyncio.coroutine
-def test_add_route_root(router):
+async def test_add_route_root(router):
     handler = make_handler()
     router.add_route('GET', '/', handler)
     req = make_request('GET', '/')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_simple(router):
+async def test_add_route_simple(router):
     handler = make_handler()
     router.add_route('GET', '/handler/to/path', handler)
     req = make_request('GET', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_with_matchdict(router):
+async def test_add_with_matchdict(router):
     handler = make_handler()
     router.add_route('GET', '/handler/{to}', handler)
     req = make_request('GET', '/handler/tail')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {'to': 'tail'} == info
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_get_shortcut(router):
+async def test_add_with_matchdict_with_colon(router):
+    handler = make_handler()
+    router.add_route('GET', '/handler/{to}', handler)
+    req = make_request('GET', '/handler/1:2:3')
+    info = await router.resolve(req)
+    assert info is not None
+    assert {'to': '1:2:3'} == info
+    assert handler is info.handler
+    assert info.route.name is None
+
+
+async def test_add_route_with_add_get_shortcut(router):
     handler = make_handler()
     router.add_get('/handler/to/path', handler)
     req = make_request('GET', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_post_shortcut(router):
+async def test_add_route_with_add_post_shortcut(router):
     handler = make_handler()
     router.add_post('/handler/to/path', handler)
     req = make_request('POST', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_put_shortcut(router):
+async def test_add_route_with_add_put_shortcut(router):
     handler = make_handler()
     router.add_put('/handler/to/path', handler)
     req = make_request('PUT', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_patch_shortcut(router):
+async def test_add_route_with_add_patch_shortcut(router):
     handler = make_handler()
     router.add_patch('/handler/to/path', handler)
     req = make_request('PATCH', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_delete_shortcut(router):
+async def test_add_route_with_add_delete_shortcut(router):
     handler = make_handler()
     router.add_delete('/handler/to/path', handler)
     req = make_request('DELETE', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_route_with_add_head_shortcut(router):
+async def test_add_route_with_add_head_shortcut(router):
     handler = make_handler()
     router.add_head('/handler/to/path', handler)
     req = make_request('HEAD', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 0 == len(info)
     assert handler is info.handler
     assert info.route.name is None
 
 
-@asyncio.coroutine
-def test_add_with_name(router):
+async def test_add_with_name(router):
     handler = make_handler()
     router.add_route('GET', '/handler/to/path', handler,
                      name='name')
     req = make_request('GET', '/handler/to/path')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert 'name' == info.route.name
 
 
-@asyncio.coroutine
-def test_add_with_tailing_slash(router):
+async def test_add_with_tailing_slash(router):
     handler = make_handler()
     router.add_route('GET', '/handler/to/path/', handler)
     req = make_request('GET', '/handler/to/path/')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {} == info
     assert handler is info.handler
@@ -229,61 +229,57 @@ def test_add_url_invalid4(router):
         router.add_route('post', '/post/{id"}', handler)
 
 
-@asyncio.coroutine
-def test_add_url_escaping(router):
+async def test_add_url_escaping(router):
     handler = make_handler()
     router.add_route('GET', '/+$', handler)
 
     req = make_request('GET', '/+$')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert handler is info.handler
 
 
-@asyncio.coroutine
-def test_any_method(router):
+async def test_any_method(router):
     handler = make_handler()
     route = router.add_route(hdrs.METH_ANY, '/', handler)
 
     req = make_request('GET', '/')
-    info1 = yield from router.resolve(req)
+    info1 = await router.resolve(req)
     assert info1 is not None
     assert route is info1.route
 
     req = make_request('POST', '/')
-    info2 = yield from router.resolve(req)
+    info2 = await router.resolve(req)
     assert info2 is not None
 
     assert info1.route is info2.route
 
 
-@asyncio.coroutine
-def test_match_second_result_in_table(router):
+async def test_match_second_result_in_table(router):
     handler1 = make_handler()
     handler2 = make_handler()
     router.add_route('GET', '/h1', handler1)
     router.add_route('POST', '/h2', handler2)
     req = make_request('POST', '/h2')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {} == info
     assert handler2 is info.handler
 
 
-@asyncio.coroutine
-def test_raise_method_not_allowed(router):
+async def test_raise_method_not_allowed(router):
     handler1 = make_handler()
     handler2 = make_handler()
     router.add_route('GET', '/', handler1)
     router.add_route('POST', '/', handler2)
     req = make_request('PUT', '/')
 
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert isinstance(match_info.route, SystemRoute)
     assert {} == match_info
 
     with pytest.raises(HTTPMethodNotAllowed) as ctx:
-        yield from match_info.handler(req)
+        await match_info.handler(req)
 
     exc = ctx.value
     assert 'PUT' == exc.method
@@ -291,18 +287,17 @@ def test_raise_method_not_allowed(router):
     assert {'POST', 'GET'} == exc.allowed_methods
 
 
-@asyncio.coroutine
-def test_raise_method_not_found(router):
+async def test_raise_method_not_found(router):
     handler = make_handler()
     router.add_route('GET', '/a', handler)
     req = make_request('GET', '/b')
 
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert isinstance(match_info.route, SystemRoute)
     assert {} == match_info
 
     with pytest.raises(HTTPNotFound) as ctx:
-        yield from match_info.handler(req)
+        await match_info.handler(req)
 
     exc = ctx.value
     assert 404 == exc.status
@@ -323,8 +318,8 @@ def test_route_plain(router):
     handler = make_handler()
     route = router.add_route('GET', '/get', handler, name='name')
     route2 = next(iter(router['name']))
-    url = route2.url()
-    assert '/get' == url
+    url = route2.url_for()
+    assert '/get' == str(url)
     assert route is route2
 
 
@@ -339,29 +334,120 @@ def test_route_dynamic(router):
                              name='name')
 
     route2 = next(iter(router['name']))
-    url = route2.url(parts={'name': 'John'})
-    assert '/get/John' == url
+    url = route2.url_for(name='John')
+    assert '/get/John' == str(url)
     assert route is route2
 
 
-def test_route_with_qs(router):
-    handler = make_handler()
-    router.add_route('GET', '/get', handler, name='name')
-
-    url = router['name'].url(query=[('a', 'b'), ('c', '1')])
-    assert '/get?a=b&c=1' == url
-
-
 def test_add_static(router):
     resource = router.add_static('/st',
                                  os.path.dirname(aiohttp.__file__),
                                  name='static')
     assert router['static'] is resource
-    url = resource.url(filename='/dir/a.txt')
-    assert '/st/dir/a.txt' == url
+    url = resource.url_for(filename='/dir/a.txt')
+    assert '/st/dir/a.txt' == str(url)
     assert len(resource) == 2
 
 
+def test_add_static_append_version(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url_for(filename='/data.unknown_mime_type',
+                           append_version=True)
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == str(url)
+
+
+def test_add_static_append_version_set_from_constructor(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 append_version=True,
+                                 name='static')
+    url = resource.url_for(filename='/data.unknown_mime_type')
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == str(url)
+
+
+def test_add_static_append_version_override_constructor(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 append_version=True,
+                                 name='static')
+    url = resource.url_for(filename='/data.unknown_mime_type',
+                           append_version=False)
+    expect_url = '/st/data.unknown_mime_type'
+    assert expect_url == str(url)
+
+
+def test_add_static_append_version_filename_without_slash(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url_for(filename='data.unknown_mime_type',
+                           append_version=True)
+    expect_url = '/st/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == str(url)
+
+
+def test_add_static_append_version_non_exists_file(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url_for(filename='/non_exists_file', append_version=True)
+    assert '/st/non_exists_file' == str(url)
+
+
+def test_add_static_append_version_non_exists_file_without_slash(router):
+    resource = router.add_static('/st',
+                                 os.path.dirname(__file__),
+                                 name='static')
+    url = resource.url_for(filename='non_exists_file', append_version=True)
+    assert '/st/non_exists_file' == str(url)
+
+
+def test_add_static_append_version_follow_symlink(router, tmpdir):
+    """
+    Tests the access to a symlink, in static folder with apeend_version
+    """
+    tmp_dir_path = str(tmpdir)
+    symlink_path = os.path.join(tmp_dir_path, 'append_version_symlink')
+    symlink_target_path = os.path.dirname(__file__)
+    os.symlink(symlink_target_path, symlink_path, True)
+
+    # Register global static route:
+    resource = router.add_static('/st', tmp_dir_path, follow_symlinks=True,
+                                 append_version=True)
+
+    url = resource.url_for(
+        filename='/append_version_symlink/data.unknown_mime_type')
+
+    expect_url = '/st/append_version_symlink/data.unknown_mime_type?' \
+                 'v=aUsn8CHEhhszc81d28QmlcBW0KQpfS2F4trgQKhOYd8%3D'
+    assert expect_url == str(url)
+
+
+def test_add_static_append_version_not_follow_symlink(router, tmpdir):
+    """
+    Tests the access to a symlink, in static folder with apeend_version
+    """
+    tmp_dir_path = str(tmpdir)
+    symlink_path = os.path.join(tmp_dir_path, 'append_version_symlink')
+    symlink_target_path = os.path.dirname(__file__)
+    os.symlink(symlink_target_path, symlink_path, True)
+
+    # Register global static route:
+    resource = router.add_static('/st', tmp_dir_path, follow_symlinks=False,
+                                 append_version=True)
+
+    filename = '/append_version_symlink/data.unknown_mime_type'
+    url = resource.url_for(filename=filename)
+    assert '/st/append_version_symlink/data.unknown_mime_type' == str(url)
+
+
 def test_plain_not_match(router):
     handler = make_handler()
     router.add_route('GET', '/get/path', handler, name='name')
@@ -376,12 +462,11 @@ def test_dynamic_not_match(router):
     assert route._match('/another/path') is None
 
 
-@asyncio.coroutine
-def test_static_not_match(router):
+async def test_static_not_match(router):
     router.add_static('/pre', os.path.dirname(aiohttp.__file__),
                       name='name')
     resource = router['name']
-    ret = yield from resource.resolve(
+    ret = await resource.resolve(
         make_mocked_request('GET', '/another/path'))
     assert (None, set()) == ret
 
@@ -433,57 +518,53 @@ def test_static_remove_trailing_slash(router):
     assert '/prefix' == route._prefix
 
 
-@asyncio.coroutine
-def test_add_route_with_re(router):
+async def test_add_route_with_re(router):
     handler = make_handler()
     router.add_route('GET', r'/handler/{to:\d+}', handler)
 
     req = make_request('GET', '/handler/1234')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {'to': '1234'} == info
 
     router.add_route('GET', r'/handler/{name}.html', handler)
     req = make_request('GET', '/handler/test.html')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert {'name': 'test'} == info
 
 
-@asyncio.coroutine
-def test_add_route_with_re_and_slashes(router):
+async def test_add_route_with_re_and_slashes(router):
     handler = make_handler()
     router.add_route('GET', r'/handler/{to:[^/]+/?}', handler)
     req = make_request('GET', '/handler/1234/')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {'to': '1234/'} == info
 
     router.add_route('GET', r'/handler/{to:.+}', handler)
     req = make_request('GET', '/handler/1234/5/6/7')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {'to': '1234/5/6/7'} == info
 
 
-@asyncio.coroutine
-def test_add_route_with_re_not_match(router):
+async def test_add_route_with_re_not_match(router):
     handler = make_handler()
     router.add_route('GET', r'/handler/{to:\d+}', handler)
 
     req = make_request('GET', '/handler/tail')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert isinstance(match_info.route, SystemRoute)
     assert {} == match_info
     with pytest.raises(HTTPNotFound):
-        yield from match_info.handler(req)
+        await match_info.handler(req)
 
 
-@asyncio.coroutine
-def test_add_route_with_re_including_slashes(router):
+async def test_add_route_with_re_including_slashes(router):
     handler = make_handler()
     router.add_route('GET', r'/handler/{to:.+}/tail', handler)
     req = make_request('GET', '/handler/re/with/slashes/tail')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info is not None
     assert {'to': 're/with/slashes'} == info
 
@@ -493,8 +574,11 @@ def test_add_route_with_invalid_re(router):
     with pytest.raises(ValueError) as ctx:
         router.add_route('GET', r'/handler/{to:+++}', handler)
     s = str(ctx.value)
-    assert s.startswith(
-        "Bad pattern '\/handler\/(?P<to>+++)': nothing to repeat")
+    assert s.startswith("Bad pattern '" +
+                        PATH_SEP +
+                        "handler" +
+                        PATH_SEP +
+                        "(?P<to>+++)': nothing to repeat")
     assert ctx.value.__cause__ is None
 
 
@@ -503,8 +587,8 @@ def test_route_dynamic_with_regex_spec(router):
     route = router.add_route('GET', '/get/{num:^\d+}', handler,
                              name='name')
 
-    url = route.url(parts={'num': '123'})
-    assert '/get/123' == url
+    url = route.url_for(num='123')
+    assert '/get/123' == str(url)
 
 
 def test_route_dynamic_with_regex_spec_and_trailing_slash(router):
@@ -512,39 +596,53 @@ def test_route_dynamic_with_regex_spec_and_trailing_slash(router):
     route = router.add_route('GET', '/get/{num:^\d+}/', handler,
                              name='name')
 
-    url = route.url(parts={'num': '123'})
-    assert '/get/123/' == url
+    url = route.url_for(num='123')
+    assert '/get/123/' == str(url)
 
 
 def test_route_dynamic_with_regex(router):
     handler = make_handler()
     route = router.add_route('GET', r'/{one}/{two:.+}', handler)
 
-    url = route.url(parts={'one': 1, 'two': 2})
-    assert '/1/2' == url
+    url = route.url_for(one='1', two='2')
+    assert '/1/2' == str(url)
 
 
-@asyncio.coroutine
-def test_regular_match_info(router):
+def test_route_dynamic_quoting(router):
+    handler = make_handler()
+    route = router.add_route('GET', r'/{arg}', handler)
+
+    url = route.url_for(arg='1 2/текст')
+    assert '/1%202/%D1%82%D0%B5%D0%BA%D1%81%D1%82' == str(url)
+
+
+async def test_regular_match_info(router):
     handler = make_handler()
     router.add_route('GET', '/get/{name}', handler)
 
     req = make_request('GET', '/get/john')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert {'name': 'john'} == match_info
     assert re.match("<MatchInfo {'name': 'john'}: .+<Dynamic.+>>",
                     repr(match_info))
 
 
-@asyncio.coroutine
-def test_not_found_repr(router):
+async def test_match_info_with_plus(router):
+    handler = make_handler()
+    router.add_route('GET', '/get/{version}', handler)
+
+    req = make_request('GET', '/get/1.0+test')
+    match_info = await router.resolve(req)
+    assert {'version': '1.0+test'} == match_info
+
+
+async def test_not_found_repr(router):
     req = make_request('POST', '/path/to')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert "<MatchInfoError 404: Not Found>" == repr(match_info)
 
 
-@asyncio.coroutine
-def test_not_allowed_repr(router):
+async def test_not_allowed_repr(router):
     handler = make_handler()
     router.add_route('GET', '/path/to', handler)
 
@@ -552,19 +650,18 @@ def test_not_allowed_repr(router):
     router.add_route('POST', '/path/to', handler2)
 
     req = make_request('PUT', '/path/to')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert "<MatchInfoError 405: Method Not Allowed>" == repr(match_info)
 
 
 def test_default_expect_handler(router):
     route = router.add_route('GET', '/', make_handler())
-    assert route._expect_handler is _defaultExpectHandler
+    assert route._expect_handler is _default_expect_handler
 
 
 def test_custom_expect_handler_plain(router):
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         pass
 
     route = router.add_route(
@@ -575,8 +672,7 @@ def test_custom_expect_handler_plain(router):
 
 def test_custom_expect_handler_dynamic(router):
 
-    @asyncio.coroutine
-    def handler(request):
+    async def handler(request):
         pass
 
     route = router.add_route(
@@ -595,42 +691,38 @@ def test_expect_handler_non_coroutine(router):
                          expect_handler=handler)
 
 
-@asyncio.coroutine
-def test_dynamic_match_non_ascii(router):
+async def test_dynamic_match_non_ascii(router):
     handler = make_handler()
     router.add_route('GET', '/{var}', handler)
     req = make_request(
         'GET',
         '/%D1%80%D1%83%D1%81%20%D1%82%D0%B5%D0%BA%D1%81%D1%82')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert {'var': 'рус текст'} == match_info
 
 
-@asyncio.coroutine
-def test_dynamic_match_with_static_part(router):
+async def test_dynamic_match_with_static_part(router):
     handler = make_handler()
     router.add_route('GET', '/{name}.html', handler)
     req = make_request('GET', '/file.html')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert {'name': 'file'} == match_info
 
 
-@asyncio.coroutine
-def test_dynamic_match_two_part2(router):
+async def test_dynamic_match_two_part2(router):
     handler = make_handler()
     router.add_route('GET', '/{name}.{ext}', handler)
     req = make_request('GET', '/file.html')
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert {'name': 'file', 'ext': 'html'} == match_info
 
 
-@asyncio.coroutine
-def test_dynamic_match_unquoted_path(router):
+async def test_dynamic_match_unquoted_path(router):
     handler = make_handler()
     router.add_route('GET', '/{path}/{subpath}', handler)
     resource_id = 'my%2Fpath%7Cwith%21some%25strange%24characters'
     req = make_request('GET', '/path/{0}'.format(resource_id))
-    match_info = yield from router.resolve(req)
+    match_info = await router.resolve(req)
     assert match_info == {
         'path': 'path',
         'subpath': unquote(resource_id)
@@ -706,9 +798,11 @@ def test_named_resources(router):
 
 
 def test_resource_iter(router):
+    async def handler(request):
+        pass
     resource = router.add_resource('/path')
-    r1 = resource.add_route('GET', lambda req: None)
-    r2 = resource.add_route('POST', lambda req: None)
+    r1 = resource.add_route('GET', handler)
+    r2 = resource.add_route('POST', handler)
     assert 2 == len(resource)
     assert [r1, r2] == list(resource)
 
@@ -731,73 +825,77 @@ def test_view_route(router):
 
 
 def test_resource_route_match(router):
+    async def handler(request):
+        pass
     resource = router.add_resource('/path')
-    route = resource.add_route('GET', lambda req: None)
+    route = resource.add_route('GET', handler)
     assert {} == route.resource._match('/path')
 
 
 def test_error_on_double_route_adding(router):
+    async def handler(request):
+        pass
     resource = router.add_resource('/path')
 
-    resource.add_route('GET', lambda: None)
+    resource.add_route('GET', handler)
     with pytest.raises(RuntimeError):
-        resource.add_route('GET', lambda: None)
+        resource.add_route('GET', handler)
 
 
 def test_error_on_adding_route_after_wildcard(router):
+    async def handler(request):
+        pass
     resource = router.add_resource('/path')
 
-    resource.add_route('*', lambda: None)
+    resource.add_route('*', handler)
     with pytest.raises(RuntimeError):
-        resource.add_route('GET', lambda: None)
+        resource.add_route('GET', handler)
 
 
-@asyncio.coroutine
-def test_http_exception_is_none_when_resolved(router):
+async def test_http_exception_is_none_when_resolved(router):
     handler = make_handler()
     router.add_route('GET', '/', handler)
     req = make_request('GET', '/')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.http_exception is None
 
 
-@asyncio.coroutine
-def test_http_exception_is_not_none_when_not_resolved(router):
+async def test_http_exception_is_not_none_when_not_resolved(router):
     handler = make_handler()
     router.add_route('GET', '/', handler)
     req = make_request('GET', '/abc')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.http_exception.status == 404
 
 
-@asyncio.coroutine
-def test_match_info_get_info_plain(router):
+async def test_match_info_get_info_plain(router):
     handler = make_handler()
     router.add_route('GET', '/', handler)
     req = make_request('GET', '/')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.get_info() == {'path': '/'}
 
 
-@asyncio.coroutine
-def test_match_info_get_info_dynamic(router):
+async def test_match_info_get_info_dynamic(router):
     handler = make_handler()
     router.add_route('GET', '/{a}', handler)
     req = make_request('GET', '/value')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.get_info() == {
-        'pattern': re.compile('\\/(?P<a>[^{}/]+)'),
+        'pattern': re.compile(PATH_SEP+'(?P<a>[^{}/]+)'),
         'formatter': '/{a}'}
 
 
-@asyncio.coroutine
-def test_match_info_get_info_dynamic2(router):
+async def test_match_info_get_info_dynamic2(router):
     handler = make_handler()
     router.add_route('GET', '/{a}/{b}', handler)
     req = make_request('GET', '/path/to')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.get_info() == {
-        'pattern': re.compile('\\/(?P<a>[^{}/]+)\\/(?P<b>[^{}/]+)'),
+        'pattern': re.compile(PATH_SEP +
+                              '(?P<a>[^{}/]+)' +
+                              PATH_SEP +
+                              '(?P<b>[^{}/]+)'),
         'formatter': '/{a}/{b}'}
 
 
@@ -808,12 +906,11 @@ def test_static_resource_get_info(router):
                                    'prefix': '/st'}
 
 
-@asyncio.coroutine
-def test_system_route_get_info(router):
+async def test_system_route_get_info(router):
     handler = make_handler()
     router.add_route('GET', '/', handler)
     req = make_request('GET', '/abc')
-    info = yield from router.resolve(req)
+    info = await router.resolve(req)
     assert info.get_info()['http_exception'].status == 404
 
 
@@ -860,30 +957,27 @@ def test_static_route_points_to_file(router):
         router.add_static('/st', here)
 
 
-@asyncio.coroutine
-def test_404_for_static_resource(router):
+async def test_404_for_static_resource(router):
     resource = router.add_static('/st',
                                  os.path.dirname(aiohttp.__file__))
-    ret = yield from resource.resolve(
+    ret = await resource.resolve(
         make_mocked_request('GET', '/unknown/path'))
     assert (None, set()) == ret
 
 
-@asyncio.coroutine
-def test_405_for_resource_adapter(router):
+async def test_405_for_resource_adapter(router):
     resource = router.add_static('/st',
                                  os.path.dirname(aiohttp.__file__))
-    ret = yield from resource.resolve(
+    ret = await resource.resolve(
         make_mocked_request('POST', '/st/abc.py'))
     assert (None, {'HEAD', 'GET'}) == ret
 
 
-@asyncio.coroutine
-def test_check_allowed_method_for_found_resource(router):
+async def test_check_allowed_method_for_found_resource(router):
     handler = make_handler()
     resource = router.add_resource('/')
     resource.add_route('GET', handler)
-    ret = yield from resource.resolve(make_mocked_request('GET', '/'))
+    ret = await resource.resolve(make_mocked_request('GET', '/'))
     assert ret[0] is not None
     assert {'GET'} == ret[1]
 
@@ -907,46 +1001,39 @@ def test_url_for_in_resource_route(router):
     assert URL('/get/John') == route.url_for(name='John')
 
 
-def test_subapp_get_info(router, loop):
-    subapp = web.Application(loop=loop)
-    resource = router.add_subapp('/pre', subapp)
+def test_subapp_get_info(app, loop):
+    subapp = web.Application()
+    resource = subapp.add_subapp('/pre', subapp)
     assert resource.get_info() == {'prefix': '/pre', 'app': subapp}
 
 
-def test_subapp_url(router, loop):
-    subapp = web.Application(loop=loop)
-    resource = router.add_subapp('/pre', subapp)
-    with pytest.raises(RuntimeError):
-        resource.url()
-
-
-def test_subapp_url_for(router, loop):
-    subapp = web.Application(loop=loop)
-    resource = router.add_subapp('/pre', subapp)
+def test_subapp_url_for(app, loop):
+    subapp = web.Application()
+    resource = app.add_subapp('/pre', subapp)
     with pytest.raises(RuntimeError):
         resource.url_for()
 
 
-def test_subapp_repr(router, loop):
-    subapp = web.Application(loop=loop)
-    resource = router.add_subapp('/pre', subapp)
+def test_subapp_repr(app, loop):
+    subapp = web.Application()
+    resource = app.add_subapp('/pre', subapp)
     assert repr(resource).startswith(
         '<PrefixedSubAppResource /pre -> <Application')
 
 
-def test_subapp_len(router, loop):
-    subapp = web.Application(loop=loop)
-    subapp.router.add_get('/', make_handler())
+def test_subapp_len(app, loop):
+    subapp = web.Application()
+    subapp.router.add_get('/', make_handler(), allow_head=False)
     subapp.router.add_post('/', make_handler())
-    resource = router.add_subapp('/pre', subapp)
+    resource = app.add_subapp('/pre', subapp)
     assert len(resource) == 2
 
 
-def test_subapp_iter(router, loop):
-    subapp = web.Application(loop=loop)
-    r1 = subapp.router.add_get('/', make_handler())
+def test_subapp_iter(app, loop):
+    subapp = web.Application()
+    r1 = subapp.router.add_get('/', make_handler(), allow_head=False)
     r2 = subapp.router.add_post('/', make_handler())
-    resource = router.add_subapp('/pre', subapp)
+    resource = app.add_subapp('/pre', subapp)
     assert list(resource) == [r1, r2]
 
 
@@ -961,11 +1048,18 @@ def test_frozen_router(router):
         router.add_get('/', make_handler())
 
 
-def test_frozen_router_subapp(loop, router):
-    subapp = web.Application(loop=loop)
+def test_frozen_router_subapp(app, loop):
+    subapp = web.Application()
     subapp.freeze()
     with pytest.raises(RuntimeError):
-        router.add_subapp('/', subapp)
+        app.add_subapp('/', subapp)
+
+
+def test_frozen_app_on_subapp(app, loop):
+    app.freeze()
+    subapp = web.Application()
+    with pytest.raises(RuntimeError):
+        app.add_subapp('/', subapp)
 
 
 def test_set_options_route(router):
@@ -991,20 +1085,19 @@ def test_dynamic_url_with_name_started_from_undescore(router):
     assert URL('/get/John') == route.url_for(_name='John')
 
 
-def test_cannot_add_subapp_with_empty_prefix(router, loop):
-    subapp = web.Application(loop=loop)
+def test_cannot_add_subapp_with_empty_prefix(app, loop):
+    subapp = web.Application()
     with pytest.raises(ValueError):
-        router.add_subapp('', subapp)
+        app.add_subapp('', subapp)
 
 
-def test_cannot_add_subapp_with_slash_prefix(router, loop):
-    subapp = web.Application(loop=loop)
+def test_cannot_add_subapp_with_slash_prefix(app, loop):
+    subapp = web.Application()
     with pytest.raises(ValueError):
-        router.add_subapp('/', subapp)
+        app.add_subapp('/', subapp)
 
 
-@asyncio.coroutine
-def test_convert_empty_path_to_slash_on_freezing(router):
+async def test_convert_empty_path_to_slash_on_freezing(router):
     handler = make_handler()
     route = router.add_get('', handler)
     resource = route.resource
@@ -1013,7 +1106,9 @@ def test_convert_empty_path_to_slash_on_freezing(router):
     assert resource.get_info() == {'path': '/'}
 
 
-def test_add_to_non_initialized_router():
-    router = UrlDispatcher()
-    with pytest.raises(RuntimeError):
-        router.add_get('/', make_handler())
+def test_deprecate_non_coroutine(router):
+    def handler(request):
+        pass
+
+    with pytest.warns(DeprecationWarning):
+        router.add_route('GET', '/handler', handler)
