@@ -1,11 +1,10 @@
-import asyncio
 import collections
 import re
 from unittest import mock
 
 import pytest
 
-from aiohttp import signals, web
+from aiohttp import helpers, signals, web
 from aiohttp.test_utils import make_mocked_request
 
 
@@ -21,13 +20,26 @@ def request(buf):
     writer = mock.Mock()
     writer.drain.return_value = ()
 
-    def append(data):
+    def append(data=b''):
         buf.extend(data)
+        return helpers.noop()
+
+    def write_headers(status_line, headers):
+        headers = status_line + ''.join(
+            [k + ': ' + v + '\r\n' for k, v in headers.items()])
+        headers = headers.encode('utf-8') + b'\r\n'
+        buf.extend(headers)
+
+    writer.buffer_data.side_effect = append
     writer.write.side_effect = append
+    writer.write_eof.side_effect = append
+    writer.write_headers.side_effect = write_headers
+
     app = mock.Mock()
     app._debug = False
     app.on_response_prepare = signals.Signal(app)
-    req = make_mocked_request(method, path, app=app, writer=writer)
+    app.on_response_prepare.freeze()
+    req = make_mocked_request(method, path, app=app, payload_writer=writer)
     return req
 
 
@@ -41,11 +53,10 @@ def test_all_http_exceptions_exported():
             assert name in web.__all__
 
 
-@asyncio.coroutine
-def test_HTTPOk(buf, request):
+async def test_HTTPOk(buf, request):
     resp = web.HTTPOk()
-    yield from resp.prepare(request)
-    yield from resp.write_eof()
+    await resp.prepare(request)
+    await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match(('HTTP/1.1 200 OK\r\n'
                      'Content-Type: text/plain; charset=utf-8\r\n'
@@ -75,18 +86,17 @@ def test_terminal_classes_has_status_code():
     assert 1 == codes.most_common(1)[0][1]
 
 
-@asyncio.coroutine
-def test_HTTPFound(buf, request):
+async def test_HTTPFound(buf, request):
     resp = web.HTTPFound(location='/redirect')
     assert '/redirect' == resp.location
     assert '/redirect' == resp.headers['location']
-    yield from resp.prepare(request)
-    yield from resp.write_eof()
+    await resp.prepare(request)
+    await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 302 Found\r\n'
                     'Content-Type: text/plain; charset=utf-8\r\n'
-                    'Content-Length: 10\r\n'
                     'Location: /redirect\r\n'
+                    'Content-Length: 10\r\n'
                     'Date: .+\r\n'
                     'Server: .+\r\n\r\n'
                     '302: Found', txt)
@@ -100,19 +110,18 @@ def test_HTTPFound_empty_location():
         web.HTTPFound(location=None)
 
 
-@asyncio.coroutine
-def test_HTTPMethodNotAllowed(buf, request):
+async def test_HTTPMethodNotAllowed(buf, request):
     resp = web.HTTPMethodNotAllowed('get', ['POST', 'PUT'])
     assert 'GET' == resp.method
     assert ['POST', 'PUT'] == resp.allowed_methods
     assert 'POST,PUT' == resp.headers['allow']
-    yield from resp.prepare(request)
-    yield from resp.write_eof()
+    await resp.prepare(request)
+    await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 405 Method Not Allowed\r\n'
                     'Content-Type: text/plain; charset=utf-8\r\n'
-                    'Content-Length: 23\r\n'
                     'Allow: POST,PUT\r\n'
+                    'Content-Length: 23\r\n'
                     'Date: .+\r\n'
                     'Server: .+\r\n\r\n'
                     '405: Method Not Allowed', txt)
