@@ -1,4 +1,4 @@
-import asyncio
+import collections
 import datetime
 import json
 import re
@@ -7,28 +7,108 @@ from unittest import mock
 import pytest
 from multidict import CIMultiDict
 
-from aiohttp import hdrs, signals
-from aiohttp.protocol import HttpVersion, HttpVersion10, HttpVersion11
-from aiohttp.test_utils import make_mocked_request
+from aiohttp import HttpVersion, HttpVersion10, HttpVersion11, hdrs, signals
+from aiohttp.payload import BytesPayload
+from aiohttp.test_utils import make_mocked_coro, make_mocked_request
 from aiohttp.web import ContentCoding, Response, StreamResponse, json_response
 
 
 def make_request(method, path, headers=CIMultiDict(),
-                 version=HttpVersion11, **kwargs):
+                 version=HttpVersion11, on_response_prepare=None, **kwargs):
     app = kwargs.pop('app', None) or mock.Mock()
     app._debug = False
-    app.on_response_prepare = signals.Signal(app)
-    writer = kwargs.pop('writer', None) or mock.Mock()
+    if on_response_prepare is None:
+        on_response_prepare = signals.Signal(app)
+    app.on_response_prepare = on_response_prepare
+    app.on_response_prepare.freeze()
+    protocol = kwargs.pop('protocol', None) or mock.Mock()
     return make_mocked_request(method, path, headers,
-                               version=version, writer=writer,
+                               version=version, protocol=protocol,
                                app=app, **kwargs)
 
 
+@pytest.fixture
+def buf():
+    return bytearray()
+
+
+@pytest.fixture
+def writer(buf):
+    writer = mock.Mock()
+
+    def acquire(cb):
+        cb(writer.transport)
+
+    def buffer_data(chunk):
+        buf.extend(chunk)
+
+    def write(chunk):
+        buf.extend(chunk)
+
+    def write_headers(status_line, headers):
+        headers = status_line + ''.join(
+            [k + ': ' + v + '\r\n' for k, v in headers.items()])
+        headers = headers.encode('utf-8') + b'\r\n'
+        buf.extend(headers)
+
+    async def write_eof(chunk=b''):
+        buf.extend(chunk)
+
+    writer.acquire.side_effect = acquire
+    writer.transport.write.side_effect = write
+    writer.write.side_effect = write
+    writer.write_eof.side_effect = write_eof
+    writer.write_headers.side_effect = write_headers
+    writer.buffer_data.side_effect = buffer_data
+    writer.drain.return_value = ()
+
+    return writer
+
+
 def test_stream_response_ctor():
     resp = StreamResponse()
     assert 200 == resp.status
     assert resp.keep_alive is None
 
+    assert resp.task is None
+
+    req = mock.Mock()
+    resp._req = req
+    assert resp.task is req.task
+
+
+def test_stream_response_hashable():
+    # should not raise exception
+    hash(StreamResponse())
+
+
+def test_stream_response_is_mutable_mapping():
+    resp = StreamResponse()
+    assert isinstance(resp, collections.MutableMapping)
+    resp['key'] = 'value'
+    assert 'value' == resp['key']
+
+
+def test_stream_response_delitem():
+    resp = StreamResponse()
+    resp['key'] = 'value'
+    del resp['key']
+    assert 'key' not in resp
+
+
+def test_stream_response_len():
+    resp = StreamResponse()
+    assert len(resp) == 0
+    resp['key'] = 'value'
+    assert len(resp) == 1
+
+
+def test_request_iter():
+    resp = StreamResponse()
+    resp['key'] = 'value'
+    resp['key2'] = 'value2'
+    assert set(resp) == {'key', 'key2'}
+
 
 def test_content_length():
     resp = StreamResponse()
@@ -42,6 +122,14 @@ def test_content_length_setter():
     assert 234 == resp.content_length
 
 
+def test_content_length_setter_with_enable_chunked_encoding():
+    resp = StreamResponse()
+
+    resp.enable_chunked_encoding()
+    with pytest.raises(RuntimeError):
+        resp.content_length = 234
+
+
 def test_drop_content_length_header_on_setting_len_to_None():
     resp = StreamResponse()
 
@@ -146,28 +234,26 @@ def test_last_modified_reset():
     assert resp.last_modified is None
 
 
-@asyncio.coroutine
-def test_start():
-    req = make_request('GET', '/')
+async def test_start():
+    req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
     assert resp.keep_alive is None
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
+    msg = await resp.prepare(req)
 
-        assert msg.send_headers.called
-        msg2 = yield from resp.prepare(req)
-        assert msg is msg2
+    assert msg.write_headers.called
+    msg2 = await resp.prepare(req)
+    assert msg is msg2
 
-        assert resp.keep_alive
+    assert resp.keep_alive
 
     req2 = make_request('GET', '/')
-    with pytest.raises(RuntimeError):
-        yield from resp.prepare(req2)
+    # with pytest.raises(RuntimeError):
+    msg3 = await resp.prepare(req2)
+    assert msg is msg3
 
 
-@asyncio.coroutine
-def test_chunked_encoding():
+async def test_chunked_encoding():
     req = make_request('GET', '/')
     resp = StreamResponse()
     assert not resp.chunked
@@ -175,42 +261,46 @@ def test_chunked_encoding():
     resp.enable_chunked_encoding()
     assert resp.chunked
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-        assert msg.chunked
+    msg = await resp.prepare(req)
+    assert msg.chunked
 
 
-@asyncio.coroutine
-def test_chunk_size():
-    req = make_request('GET', '/')
+def test_enable_chunked_encoding_with_content_length():
+    resp = StreamResponse()
+
+    resp.content_length = 234
+    with pytest.raises(RuntimeError):
+        resp.enable_chunked_encoding()
+
+
+async def test_chunk_size():
+    req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
     assert not resp.chunked
 
-    resp.enable_chunked_encoding(chunk_size=8192)
+    with pytest.warns(DeprecationWarning):
+        resp.enable_chunked_encoding(chunk_size=8192)
     assert resp.chunked
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-        assert msg.chunked
-        msg.add_chunking_filter.assert_called_with(8192)
-        assert msg.filter is not None
+    msg = await resp.prepare(req)
+    assert msg.chunked
+    assert msg.enable_chunking.called
+    assert msg.filter is not None
 
 
-@asyncio.coroutine
-def test_chunked_encoding_forbidden_for_http_10():
+async def test_chunked_encoding_forbidden_for_http_10():
     req = make_request('GET', '/', version=HttpVersion10)
     resp = StreamResponse()
     resp.enable_chunked_encoding()
 
     with pytest.raises(RuntimeError) as ctx:
-        yield from resp.prepare(req)
+        await resp.prepare(req)
     assert re.match("Using chunked encoding is forbidden for HTTP/1.0",
                     str(ctx.value))
 
 
-@asyncio.coroutine
-def test_compression_no_accept():
-    req = make_request('GET', '/')
+async def test_compression_no_accept():
+    req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
     assert not resp.chunked
 
@@ -218,14 +308,12 @@ def test_compression_no_accept():
     resp.enable_compression()
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-        assert not msg.add_compression_filter.called
+    msg = await resp.prepare(req)
+    assert not msg.enable_compression.called
 
 
-@asyncio.coroutine
-def test_force_compression_no_accept_backwards_compat():
-    req = make_request('GET', '/')
+async def test_force_compression_no_accept_backwards_compat():
+    req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
     assert not resp.chunked
 
@@ -233,28 +321,24 @@ def test_force_compression_no_accept_backwards_compat():
     resp.enable_compression(force=True)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    assert msg.add_compression_filter.called
+    msg = await resp.prepare(req)
+    assert msg.enable_compression.called
     assert msg.filter is not None
 
 
-@asyncio.coroutine
-def test_force_compression_false_backwards_compat():
-    req = make_request('GET', '/')
+async def test_force_compression_false_backwards_compat():
+    req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
 
     assert not resp.compression
     resp.enable_compression(force=False)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    assert not msg.add_compression_filter.called
+    msg = await resp.prepare(req)
+    assert not msg.enable_compression.called
 
 
-@asyncio.coroutine
-def test_compression_default_coding():
+async def test_compression_default_coding():
     req = make_request(
         'GET', '/',
         headers=CIMultiDict({hdrs.ACCEPT_ENCODING: 'gzip, deflate'}))
@@ -265,16 +349,14 @@ def test_compression_default_coding():
     resp.enable_compression()
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
+    msg = await resp.prepare(req)
 
-    msg.add_compression_filter.assert_called_with('deflate')
+    msg.enable_compression.assert_called_with('deflate')
     assert 'deflate' == resp.headers.get(hdrs.CONTENT_ENCODING)
     assert msg.filter is not None
 
 
-@asyncio.coroutine
-def test_force_compression_deflate():
+async def test_force_compression_deflate():
     req = make_request(
         'GET', '/',
         headers=CIMultiDict({hdrs.ACCEPT_ENCODING: 'gzip, deflate'}))
@@ -283,28 +365,24 @@ def test_force_compression_deflate():
     resp.enable_compression(ContentCoding.deflate)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    msg.add_compression_filter.assert_called_with('deflate')
+    msg = await resp.prepare(req)
+    msg.enable_compression.assert_called_with('deflate')
     assert 'deflate' == resp.headers.get(hdrs.CONTENT_ENCODING)
 
 
-@asyncio.coroutine
-def test_force_compression_no_accept_deflate():
+async def test_force_compression_no_accept_deflate():
     req = make_request('GET', '/')
     resp = StreamResponse()
 
     resp.enable_compression(ContentCoding.deflate)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    msg.add_compression_filter.assert_called_with('deflate')
+    msg = await resp.prepare(req)
+    msg.enable_compression.assert_called_with('deflate')
     assert 'deflate' == resp.headers.get(hdrs.CONTENT_ENCODING)
 
 
-@asyncio.coroutine
-def test_force_compression_gzip():
+async def test_force_compression_gzip():
     req = make_request(
         'GET', '/',
         headers=CIMultiDict({hdrs.ACCEPT_ENCODING: 'gzip, deflate'}))
@@ -313,111 +391,227 @@ def test_force_compression_gzip():
     resp.enable_compression(ContentCoding.gzip)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    msg.add_compression_filter.assert_called_with('gzip')
+    msg = await resp.prepare(req)
+    msg.enable_compression.assert_called_with('gzip')
     assert 'gzip' == resp.headers.get(hdrs.CONTENT_ENCODING)
 
 
-@asyncio.coroutine
-def test_force_compression_no_accept_gzip():
+async def test_force_compression_no_accept_gzip():
     req = make_request('GET', '/')
     resp = StreamResponse()
 
     resp.enable_compression(ContentCoding.gzip)
     assert resp.compression
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        msg = yield from resp.prepare(req)
-    msg.add_compression_filter.assert_called_with('gzip')
+    msg = await resp.prepare(req)
+    msg.enable_compression.assert_called_with('gzip')
     assert 'gzip' == resp.headers.get(hdrs.CONTENT_ENCODING)
 
 
-@asyncio.coroutine
-def test_delete_content_length_if_compression_enabled():
+async def test_change_content_length_if_compression_enabled():
     req = make_request('GET', '/')
     resp = Response(body=b'answer')
-    assert 6 == resp.content_length
-
     resp.enable_compression(ContentCoding.gzip)
 
-    with mock.patch('aiohttp.web_reqrep.ResponseImpl'):
-        yield from resp.prepare(req)
+    await resp.prepare(req)
+    assert resp.content_length is not None and \
+        resp.content_length != len(b'answer')
+
+
+async def test_set_content_length_if_compression_enabled():
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH in headers
+        assert headers[hdrs.CONTENT_LENGTH] == '26'
+        assert hdrs.TRANSFER_ENCODING not in headers
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/', payload_writer=writer)
+    resp = Response(body=b'answer')
+    resp.enable_compression(ContentCoding.gzip)
+
+    await resp.prepare(req)
+    assert resp.content_length == 26
+    del resp.headers[hdrs.CONTENT_LENGTH]
+    assert resp.content_length == 26
+
+
+async def test_remove_content_length_if_compression_enabled_http11():
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH not in headers
+        assert headers.get(hdrs.TRANSFER_ENCODING, '') == 'chunked'
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/', payload_writer=writer)
+    resp = StreamResponse()
+    resp.content_length = 123
+    resp.enable_compression(ContentCoding.gzip)
+    await resp.prepare(req)
     assert resp.content_length is None
 
 
-@asyncio.coroutine
-def test_write_non_byteish():
+async def test_remove_content_length_if_compression_enabled_http10():
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH not in headers
+        assert hdrs.TRANSFER_ENCODING not in headers
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/', version=HttpVersion10,
+                       payload_writer=writer)
     resp = StreamResponse()
-    yield from resp.prepare(make_request('GET', '/'))
+    resp.content_length = 123
+    resp.enable_compression(ContentCoding.gzip)
+    await resp.prepare(req)
+    assert resp.content_length is None
+
+
+async def test_force_compression_identity():
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH in headers
+        assert hdrs.TRANSFER_ENCODING not in headers
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/',
+                       payload_writer=writer)
+    resp = StreamResponse()
+    resp.content_length = 123
+    resp.enable_compression(ContentCoding.identity)
+    await resp.prepare(req)
+    assert resp.content_length == 123
+
+
+async def test_force_compression_identity_response():
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert headers[hdrs.CONTENT_LENGTH] == "6"
+        assert hdrs.TRANSFER_ENCODING not in headers
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/',
+                       payload_writer=writer)
+    resp = Response(body=b'answer')
+    resp.enable_compression(ContentCoding.identity)
+    await resp.prepare(req)
+    assert resp.content_length == 6
+
+
+async def test_remove_content_length_if_compression_enabled_on_payload_http11():  # noqa
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH not in headers
+        assert headers.get(hdrs.TRANSFER_ENCODING, '') == 'chunked'
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/', payload_writer=writer)
+    payload = BytesPayload(b'answer', headers={"X-Test-Header": "test"})
+    resp = Response(body=payload)
+    assert resp.content_length == 6
+    resp.body = payload
+    resp.enable_compression(ContentCoding.gzip)
+    await resp.prepare(req)
+    assert resp.content_length is None
+
+
+async def test_remove_content_length_if_compression_enabled_on_payload_http10():  # noqa
+    writer = mock.Mock()
+
+    def write_headers(status_line, headers):
+        assert hdrs.CONTENT_LENGTH not in headers
+        assert hdrs.TRANSFER_ENCODING not in headers
+
+    writer.write_headers.side_effect = write_headers
+    req = make_request('GET', '/', version=HttpVersion10,
+                       payload_writer=writer)
+    resp = Response(body=BytesPayload(b'answer'))
+    resp.enable_compression(ContentCoding.gzip)
+    await resp.prepare(req)
+    assert resp.content_length is None
+
+
+async def test_content_length_on_chunked():
+    req = make_request('GET', '/')
+    resp = Response(body=b'answer')
+    assert resp.content_length == 6
+    resp.enable_chunked_encoding()
+    assert resp.content_length is None
+    await resp.prepare(req)
+
+
+async def test_write_non_byteish():
+    resp = StreamResponse()
+    await resp.prepare(make_request('GET', '/'))
 
     with pytest.raises(AssertionError):
-        resp.write(123)
+        await resp.write(123)
 
 
-def test_write_before_start():
+async def test_write_before_start():
     resp = StreamResponse()
 
     with pytest.raises(RuntimeError):
-        resp.write(b'data')
+        await resp.write(b'data')
 
 
-@asyncio.coroutine
-def test_cannot_write_after_eof():
+async def test_cannot_write_after_eof():
+    resp = StreamResponse()
+    req = make_request('GET', '/')
+    await resp.prepare(req)
+
+    await resp.write(b'data')
+    await resp.write_eof()
+    req.writer.write.reset_mock()
+
+    with pytest.raises(RuntimeError):
+        await resp.write(b'next data')
+    assert not req.writer.write.called
+
+
+async def test___repr___after_eof():
+    resp = StreamResponse()
+    await resp.prepare(make_request('GET', '/'))
+
+    assert resp.prepared
+
+    await resp.write(b'data')
+    await resp.write_eof()
+    assert not resp.prepared
+    resp_repr = repr(resp)
+    assert resp_repr == '<StreamResponse OK eof>'
+
+
+async def test_cannot_write_eof_before_headers():
+    resp = StreamResponse()
+
+    with pytest.raises(AssertionError):
+        await resp.write_eof()
+
+
+async def test_cannot_write_eof_twice():
     resp = StreamResponse()
     writer = mock.Mock()
-    yield from resp.prepare(make_request('GET', '/', writer=writer))
+    resp_impl = await resp.prepare(make_request('GET', '/'))
+    resp_impl.write = make_mocked_coro(None)
+    resp_impl.write_eof = make_mocked_coro(None)
 
-    resp.write(b'data')
-    writer.drain.return_value = ()
-    yield from resp.write_eof()
-    writer.write.reset_mock()
+    await resp.write(b'data')
+    assert resp_impl.write.called
 
-    with pytest.raises(RuntimeError):
-        resp.write(b'next data')
+    await resp.write_eof()
+
+    resp_impl.write.reset_mock()
+    await resp.write_eof()
     assert not writer.write.called
 
 
-@asyncio.coroutine
-def test_cannot_write_eof_before_headers():
-    resp = StreamResponse()
-
-    with pytest.raises(RuntimeError):
-        yield from resp.write_eof()
-
-
-@asyncio.coroutine
-def test_cannot_write_eof_twice():
-    resp = StreamResponse()
-    writer = mock.Mock()
-    yield from resp.prepare(make_request('GET', '/', writer=writer))
-
-    resp.write(b'data')
-    writer.drain.return_value = ()
-    yield from resp.write_eof()
-    assert writer.write.called
-
-    writer.write.reset_mock()
-    yield from resp.write_eof()
-    assert not writer.write.called
-
-
-@asyncio.coroutine
-def test_write_returns_drain():
-    resp = StreamResponse()
-    yield from resp.prepare(make_request('GET', '/'))
-
-    assert () == resp.write(b'data')
-
-
-@asyncio.coroutine
-def test_write_returns_empty_tuple_on_empty_data():
-    resp = StreamResponse()
-    yield from resp.prepare(make_request('GET', '/'))
-
-    assert () == resp.write(b'')
-
-
 def test_force_close():
     resp = StreamResponse()
 
@@ -426,6 +620,13 @@ def test_force_close():
     assert resp.keep_alive is False
 
 
+async def test_response_output_length():
+    resp = StreamResponse()
+    await resp.prepare(make_request('GET', '/'))
+    with pytest.warns(DeprecationWarning):
+        assert resp.output_length
+
+
 def test_response_cookies():
     resp = StreamResponse()
 
@@ -505,147 +706,75 @@ def test_set_status_with_reason():
     assert "Everithing is fine!" == resp.reason
 
 
-@asyncio.coroutine
-def test_start_force_close():
+async def test_start_force_close():
     req = make_request('GET', '/')
     resp = StreamResponse()
     resp.force_close()
     assert not resp.keep_alive
 
-    msg = yield from resp.prepare(req)
+    await resp.prepare(req)
     assert not resp.keep_alive
-    assert msg.closing
 
 
-@asyncio.coroutine
-def test___repr__():
+async def test___repr__():
     req = make_request('GET', '/path/to')
     resp = StreamResponse(reason=301)
-    yield from resp.prepare(req)
+    await resp.prepare(req)
     assert "<StreamResponse 301 GET /path/to >" == repr(resp)
 
 
-def test___repr__not_started():
+def test___repr___not_prepared():
     resp = StreamResponse(reason=301)
-    assert "<StreamResponse 301 not started>" == repr(resp)
+    assert "<StreamResponse 301 not prepared>" == repr(resp)
 
 
-@asyncio.coroutine
-def test_keep_alive_http10_default():
+async def test_keep_alive_http10_default():
     req = make_request('GET', '/', version=HttpVersion10)
     resp = StreamResponse()
-    yield from resp.prepare(req)
+    await resp.prepare(req)
     assert not resp.keep_alive
 
 
-@asyncio.coroutine
-def test_keep_alive_http10_switched_on():
+async def test_keep_alive_http10_switched_on():
     headers = CIMultiDict(Connection='keep-alive')
     req = make_request('GET', '/', version=HttpVersion10, headers=headers)
     req._message = req._message._replace(should_close=False)
     resp = StreamResponse()
-    yield from resp.prepare(req)
+    await resp.prepare(req)
     assert resp.keep_alive
 
 
-@asyncio.coroutine
-def test_keep_alive_http09():
+async def test_keep_alive_http09():
     headers = CIMultiDict(Connection='keep-alive')
     req = make_request('GET', '/', version=HttpVersion(0, 9), headers=headers)
     resp = StreamResponse()
-    yield from resp.prepare(req)
+    await resp.prepare(req)
     assert not resp.keep_alive
 
 
-def test_start_twice():
+async def test_prepare_twice():
     req = make_request('GET', '/')
     resp = StreamResponse()
 
-    with pytest.warns(DeprecationWarning):
-        impl1 = resp.start(req)
-        impl2 = resp.start(req)
-        assert impl1 is impl2
+    impl1 = await resp.prepare(req)
+    impl2 = await resp.prepare(req)
+    assert impl1 is impl2
 
 
-@asyncio.coroutine
-def test_prepare_calls_signal():
+async def test_prepare_calls_signal():
     app = mock.Mock()
-    req = make_request('GET', '/', app=app)
+    sig = make_mocked_coro()
+    on_response_prepare = signals.Signal(app)
+    on_response_prepare.append(sig)
+    req = make_request('GET', '/', app=app,
+                       on_response_prepare=on_response_prepare)
     resp = StreamResponse()
 
-    sig = mock.Mock()
-    app.on_response_prepare.append(sig)
-    yield from resp.prepare(req)
+    await resp.prepare(req)
 
     sig.assert_called_with(req, resp)
 
 
-def test_get_nodelay_unprepared():
-    resp = StreamResponse()
-    with pytest.raises(RuntimeError):
-        resp.tcp_nodelay
-
-
-def test_set_nodelay_unprepared():
-    resp = StreamResponse()
-    with pytest.raises(RuntimeError):
-        resp.set_tcp_nodelay(True)
-
-
-@asyncio.coroutine
-def test_get_nodelay_prepared():
-    resp = StreamResponse()
-    writer = mock.Mock()
-    writer.tcp_nodelay = False
-    req = make_request('GET', '/', writer=writer)
-
-    yield from resp.prepare(req)
-    assert not resp.tcp_nodelay
-
-
-def test_set_nodelay_prepared():
-    resp = StreamResponse()
-    writer = mock.Mock()
-    req = make_request('GET', '/', writer=writer)
-
-    yield from resp.prepare(req)
-    resp.set_tcp_nodelay(True)
-    writer.set_tcp_nodelay.assert_called_with(True)
-
-
-def test_get_cork_unprepared():
-    resp = StreamResponse()
-    with pytest.raises(RuntimeError):
-        resp.tcp_cork
-
-
-def test_set_cork_unprepared():
-    resp = StreamResponse()
-    with pytest.raises(RuntimeError):
-        resp.set_tcp_cork(True)
-
-
-@asyncio.coroutine
-def test_get_cork_prepared():
-    resp = StreamResponse()
-    writer = mock.Mock()
-    writer.tcp_cork = False
-    req = make_request('GET', '/', writer=writer)
-
-    yield from resp.prepare(req)
-    assert not resp.tcp_cork
-
-
-def test_set_cork_prepared():
-    resp = StreamResponse()
-    writer = mock.Mock()
-    req = make_request('GET', '/', writer=writer)
-
-    yield from resp.prepare(req)
-    resp.set_tcp_cork(True)
-    writer.set_tcp_cork.assert_called_with(True)
-
-
 # Response class
 
 
@@ -655,22 +784,21 @@ def test_response_ctor():
     assert 200 == resp.status
     assert 'OK' == resp.reason
     assert resp.body is None
-    assert 0 == resp.content_length
-    assert (CIMultiDict([('CONTENT-TYPE', 'application/octet-stream'),
-                         ('CONTENT-LENGTH', '0')]) ==
-            resp.headers)
+    assert resp.content_length == 0
+    assert 'CONTENT-LENGTH' not in resp.headers
 
 
 def test_ctor_with_headers_and_status():
-    resp = Response(body=b'body', status=201, headers={'Age': '12'})
+    resp = Response(body=b'body', status=201,
+                    headers={'Age': '12', 'DATE': 'date'})
 
     assert 201 == resp.status
     assert b'body' == resp.body
+    assert resp.headers['AGE'] == '12'
+
+    resp._start(mock.Mock(version=HttpVersion11))
     assert 4 == resp.content_length
-    assert (CIMultiDict([('AGE', '12'),
-                         ('CONTENT-TYPE', 'application/octet-stream'),
-                         ('CONTENT-LENGTH', '4')]) ==
-            resp.headers)
+    assert resp.headers['CONTENT-LENGTH'] == '4'
 
 
 def test_ctor_content_type():
@@ -678,8 +806,8 @@ def test_ctor_content_type():
 
     assert 200 == resp.status
     assert 'OK' == resp.reason
-    assert (CIMultiDict([('CONTENT-TYPE', 'application/json'),
-                         ('CONTENT-LENGTH', '0')]) ==
+    assert 0 == resp.content_length
+    assert (CIMultiDict([('CONTENT-TYPE', 'application/json')]) ==
             resp.headers)
 
 
@@ -693,13 +821,17 @@ def test_ctor_text():
 
     assert 200 == resp.status
     assert 'OK' == resp.reason
+    assert 9 == resp.content_length
     assert (CIMultiDict(
-        [('CONTENT-TYPE', 'text/plain; charset=utf-8'),
-         ('CONTENT-LENGTH', '9')]) == resp.headers)
+        [('CONTENT-TYPE', 'text/plain; charset=utf-8')]) == resp.headers)
 
     assert resp.body == b'test text'
     assert resp.text == 'test text'
 
+    resp.headers['DATE'] = 'date'
+    resp._start(mock.Mock(version=HttpVersion11))
+    assert resp.headers['CONTENT-LENGTH'] == '9'
+
 
 def test_ctor_charset():
     resp = Response(text='текст', charset='koi8-r')
@@ -725,6 +857,14 @@ def test_ctor_charset_without_text():
     assert 'koi8-r' == resp.charset
 
 
+def test_ctor_content_type_with_extra():
+    resp = Response(text='test test', content_type='text/plain; version=0.0.4')
+
+    assert resp.content_type == 'text/plain'
+    assert resp.headers['content-type'] == \
+        'text/plain; version=0.0.4; charset=utf-8'
+
+
 def test_ctor_both_content_type_param_and_header_with_text():
     with pytest.raises(ValueError):
         Response(headers={'Content-Type': 'application/json'},
@@ -752,99 +892,92 @@ def test_ctor_both_charset_param_and_header():
 def test_assign_nonbyteish_body():
     resp = Response(body=b'data')
 
-    with pytest.raises(TypeError):
+    with pytest.raises(ValueError):
         resp.body = 123
     assert b'data' == resp.body
     assert 4 == resp.content_length
 
+    resp.headers['DATE'] = 'date'
+    resp._start(mock.Mock(version=HttpVersion11))
+    assert resp.headers['CONTENT-LENGTH'] == '4'
+    assert 4 == resp.content_length
+
 
 def test_assign_nonstr_text():
     resp = Response(text='test')
 
-    with pytest.raises(TypeError):
+    with pytest.raises(AssertionError):
         resp.text = b'123'
     assert b'test' == resp.body
     assert 4 == resp.content_length
 
 
-@asyncio.coroutine
-def test_send_headers_for_empty_body():
-    writer = mock.Mock()
-    req = make_request('GET', '/', writer=writer)
+def test_response_set_content_length():
+    resp = Response()
+    with pytest.raises(RuntimeError):
+        resp.content_length = 1
+
+
+async def test_send_headers_for_empty_body(buf, writer):
+    req = make_request('GET', '/', payload_writer=writer)
     resp = Response()
 
-    writer.drain.return_value = ()
-    buf = b''
-
-    def append(data):
-        nonlocal buf
-        buf += data
-
-    writer.write.side_effect = append
-
-    yield from resp.prepare(req)
-    yield from resp.write_eof()
+    await resp.prepare(req)
+    await resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 200 OK\r\n'
-                    'Content-Type: application/octet-stream\r\n'
                     'Content-Length: 0\r\n'
+                    'Content-Type: application/octet-stream\r\n'
                     'Date: .+\r\n'
                     'Server: .+\r\n\r\n', txt)
 
 
-@asyncio.coroutine
-def test_render_with_body():
-    writer = mock.Mock()
-    req = make_request('GET', '/', writer=writer)
+async def test_render_with_body(buf, writer):
+    req = make_request('GET', '/', payload_writer=writer)
     resp = Response(body=b'data')
 
-    writer.drain.return_value = ()
-    buf = b''
+    await resp.prepare(req)
+    await resp.write_eof()
 
-    def append(data):
-        nonlocal buf
-        buf += data
-
-    writer.write.side_effect = append
-
-    yield from resp.prepare(req)
-    yield from resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 200 OK\r\n'
-                    'Content-Type: application/octet-stream\r\n'
                     'Content-Length: 4\r\n'
+                    'Content-Type: application/octet-stream\r\n'
                     'Date: .+\r\n'
                     'Server: .+\r\n\r\n'
                     'data', txt)
 
 
-@asyncio.coroutine
-def test_send_set_cookie_header():
+async def test_send_set_cookie_header(buf, writer):
     resp = Response()
     resp.cookies['name'] = 'value'
+    req = make_request('GET', '/', payload_writer=writer)
 
-    writer = mock.Mock()
-    req = make_request('GET', '/', writer=writer)
-    writer.drain.return_value = ()
-    buf = b''
+    await resp.prepare(req)
+    await resp.write_eof()
 
-    def append(data):
-        nonlocal buf
-        buf += data
-
-    writer.write.side_effect = append
-
-    yield from resp.prepare(req)
-    yield from resp.write_eof()
     txt = buf.decode('utf8')
     assert re.match('HTTP/1.1 200 OK\r\n'
-                    'Content-Type: application/octet-stream\r\n'
                     'Content-Length: 0\r\n'
                     'Set-Cookie: name=value\r\n'
+                    'Content-Type: application/octet-stream\r\n'
                     'Date: .+\r\n'
                     'Server: .+\r\n\r\n', txt)
 
 
+async def test_consecutive_write_eof():
+    payload_writer = mock.Mock()
+    payload_writer.write_eof = make_mocked_coro()
+    req = make_request('GET', '/', payload_writer=payload_writer)
+    data = b'data'
+    resp = Response(body=data)
+
+    await resp.prepare(req)
+    await resp.write_eof()
+    await resp.write_eof()
+    payload_writer.write_eof.assert_called_once_with(data)
+
+
 def test_set_text_with_content_type():
     resp = Response()
     resp.content_type = "text/html"
@@ -891,25 +1024,22 @@ def test_started_when_not_started():
     assert not resp.prepared
 
 
-@asyncio.coroutine
-def test_started_when_started():
+async def test_started_when_started():
     resp = StreamResponse()
-    yield from resp.prepare(make_request('GET', '/'))
+    await resp.prepare(make_request('GET', '/'))
     assert resp.prepared
 
 
-@asyncio.coroutine
-def test_drain_before_start():
+async def test_drain_before_start():
     resp = StreamResponse()
-    with pytest.raises(RuntimeError):
-        yield from resp.drain()
+    with pytest.raises(AssertionError):
+        await resp.drain()
 
 
-@asyncio.coroutine
-def test_changing_status_after_prepare_raises():
+async def test_changing_status_after_prepare_raises():
     resp = StreamResponse()
-    yield from resp.prepare(make_request('GET', '/'))
-    with pytest.raises(RuntimeError):
+    await resp.prepare(make_request('GET', '/'))
+    with pytest.raises(AssertionError):
         resp.set_status(400)
 
 
