@@ -1,8 +1,8 @@
 from unittest import mock
 
-from aiohttp import hdrs, helpers
+from aiohttp import hdrs
 from aiohttp.test_utils import make_mocked_coro, make_mocked_request
-from aiohttp.web_fileresponse import FileResponse, SendfilePayloadWriter
+from aiohttp.web_fileresponse import FileResponse, SendfileStreamWriter
 
 
 def test_static_handle_eof(loop):
@@ -10,9 +10,9 @@ def test_static_handle_eof(loop):
     with mock.patch('aiohttp.web_fileresponse.os') as m_os:
         out_fd = 30
         in_fd = 31
-        fut = helpers.create_future(loop)
+        fut = loop.create_future()
         m_os.sendfile.return_value = 0
-        writer = SendfilePayloadWriter(fake_loop, mock.Mock())
+        writer = SendfileStreamWriter(mock.Mock(), mock.Mock(), fake_loop)
         writer._sendfile_cb(fut, out_fd, in_fd, 0, 100, fake_loop, False)
         m_os.sendfile.assert_called_with(out_fd, in_fd, 0, 100)
         assert fut.done()
@@ -26,9 +26,9 @@ def test_static_handle_again(loop):
     with mock.patch('aiohttp.web_fileresponse.os') as m_os:
         out_fd = 30
         in_fd = 31
-        fut = helpers.create_future(loop)
+        fut = loop.create_future()
         m_os.sendfile.side_effect = BlockingIOError()
-        writer = SendfilePayloadWriter(fake_loop, mock.Mock())
+        writer = SendfileStreamWriter(mock.Mock(), mock.Mock(), fake_loop)
         writer._sendfile_cb(fut, out_fd, in_fd, 0, 100, fake_loop, False)
         m_os.sendfile.assert_called_with(out_fd, in_fd, 0, 100)
         assert not fut.done()
@@ -44,10 +44,10 @@ def test_static_handle_exception(loop):
     with mock.patch('aiohttp.web_fileresponse.os') as m_os:
         out_fd = 30
         in_fd = 31
-        fut = helpers.create_future(loop)
+        fut = loop.create_future()
         exc = OSError()
         m_os.sendfile.side_effect = exc
-        writer = SendfilePayloadWriter(fake_loop, mock.Mock())
+        writer = SendfileStreamWriter(mock.Mock(), mock.Mock(), fake_loop)
         writer._sendfile_cb(fut, out_fd, in_fd, 0, 100, fake_loop, False)
         m_os.sendfile.assert_called_with(out_fd, in_fd, 0, 100)
         assert fut.done()
@@ -61,9 +61,9 @@ def test__sendfile_cb_return_on_cancelling(loop):
     with mock.patch('aiohttp.web_fileresponse.os') as m_os:
         out_fd = 30
         in_fd = 31
-        fut = helpers.create_future(loop)
+        fut = loop.create_future()
         fut.cancel()
-        writer = SendfilePayloadWriter(fake_loop, mock.Mock())
+        writer = SendfileStreamWriter(mock.Mock(), mock.Mock(), fake_loop)
         writer._sendfile_cb(fut, out_fd, in_fd, 0, 100, fake_loop, False)
         assert fut.done()
         assert not fake_loop.add_writer.called
