@@ -1,4 +1,3 @@
-import asyncio
 import functools
 import os
 import shutil
@@ -29,23 +28,25 @@ def tmp_dir_path(request):
     return tmp_dir
 
 
-@pytest.mark.parametrize("show_index,status,prefix,data",
-                         [(False, 403, '/', None),
-                          (True, 200, '/',
-                           b'<html>\n<head>\n<title>Index of /.</title>\n'
-                           b'</head>\n<body>\n<h1>Index of /.</h1>\n<ul>\n'
-                           b'<li><a href="/my_dir">my_dir/</a></li>\n'
-                           b'<li><a href="/my_file">my_file</a></li>\n'
-                           b'</ul>\n</body>\n</html>'),
-                          (True, 200, '/static',
-                           b'<html>\n<head>\n<title>Index of /.</title>\n'
-                           b'</head>\n<body>\n<h1>Index of /.</h1>\n<ul>\n'
-                           b'<li><a href="/static/my_dir">my_dir/</a></li>\n'
-                           b'<li><a href="/static/my_file">my_file</a></li>\n'
-                           b'</ul>\n</body>\n</html>')])
-@asyncio.coroutine
-def test_access_root_of_static_handler(tmp_dir_path, loop, test_client,
-                                       show_index, status, prefix, data):
+@pytest.mark.parametrize(
+    "show_index,status,prefix,data",
+    [pytest.param(False, 403, '/', None, id="index_forbidden"),
+     pytest.param(True, 200, '/',
+                  b'<html>\n<head>\n<title>Index of /.</title>\n'
+                  b'</head>\n<body>\n<h1>Index of /.</h1>\n<ul>\n'
+                  b'<li><a href="/my_dir">my_dir/</a></li>\n'
+                  b'<li><a href="/my_file">my_file</a></li>\n'
+                  b'</ul>\n</body>\n</html>',
+                  id="index_root"),
+     pytest.param(True, 200, '/static',
+                  b'<html>\n<head>\n<title>Index of /.</title>\n'
+                  b'</head>\n<body>\n<h1>Index of /.</h1>\n<ul>\n'
+                  b'<li><a href="/static/my_dir">my_dir/</a></li>\n'
+                  b'<li><a href="/static/my_file">my_file</a></li>\n'
+                  b'</ul>\n</body>\n</html>',
+                  id="index_static")])
+async def test_access_root_of_static_handler(tmp_dir_path, aiohttp_client,
+                                             show_index, status, prefix, data):
     """
     Tests the operation of static file server.
     Try to access the root of static file server, and make
@@ -68,24 +69,24 @@ def test_access_root_of_static_handler(tmp_dir_path, loop, test_client,
 
     # Register global static route:
     app.router.add_static(prefix, tmp_dir_path, show_index=show_index)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
     # Request the root of the static directory.
-    r = yield from client.get(prefix)
+    r = await client.get(prefix)
     assert r.status == status
 
     if data:
         assert r.headers['Content-Type'] == "text/html; charset=utf-8"
-        read_ = (yield from r.read())
+        read_ = (await r.read())
         assert read_ == data
 
 
-@pytest.mark.parametrize('data', ['hello world'])
-@asyncio.coroutine
-def test_follow_symlink(tmp_dir_path, loop, test_client, data):
+async def test_follow_symlink(tmp_dir_path, aiohttp_client):
     """
     Tests the access to a symlink, in static folder
     """
+    data = 'hello world'
+
     my_dir_path = os.path.join(tmp_dir_path, 'my_dir')
     os.mkdir(my_dir_path)
 
@@ -100,21 +101,20 @@ def test_follow_symlink(tmp_dir_path, loop, test_client, data):
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, follow_symlinks=True)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
     # Request the root of the static directory.
-    r = yield from client.get('/my_symlink/my_file_in_dir')
+    r = await client.get('/my_symlink/my_file_in_dir')
     assert r.status == 200
-    assert (yield from r.text()) == data
+    assert (await r.text()) == data
 
 
 @pytest.mark.parametrize('dir_name,filename,data', [
     ('', 'test file.txt', 'test text'),
     ('test dir name', 'test dir file .txt', 'test text file folder')
 ])
-@asyncio.coroutine
-def test_access_to_the_file_with_spaces(tmp_dir_path, loop, test_client,
-                                        dir_name, filename, data):
+async def test_access_to_the_file_with_spaces(tmp_dir_path, aiohttp_client,
+                                              dir_name, filename, data):
     """
     Checks operation of static files with spaces
     """
@@ -134,15 +134,14 @@ def test_access_to_the_file_with_spaces(tmp_dir_path, loop, test_client,
     url = os.path.join('/', dir_name, filename)
 
     app.router.add_static('/', tmp_dir_path)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    r = yield from client.get(url)
+    r = await client.get(url)
     assert r.status == 200
-    assert (yield from r.text()) == data
+    assert (await r.text()) == data
 
 
-@asyncio.coroutine
-def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
+async def test_access_non_existing_resource(tmp_dir_path, aiohttp_client):
     """
     Tests accessing non-existing resource
     Try to access a non-exiting resource and make sure that 404 HTTP status
@@ -152,10 +151,10 @@ def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
     # Request the root of the static directory.
-    r = yield from client.get('/non_existing_resource')
+    r = await client.get('/non_existing_resource')
     assert r.status == 404
 
 
@@ -164,49 +163,46 @@ def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
     ('/a@b', '/a@b'),
     ('/a:b', '/a%3Ab'),
 ])
-@asyncio.coroutine
-def test_url_escaping(loop, test_client, registered_path, request_url):
+async def test_url_escaping(aiohttp_client, registered_path, request_url):
     """
     Tests accessing a resource with
     """
     app = web.Application()
 
-    def handler(_):
+    async def handler(request):
         return web.Response()
     app.router.add_get(registered_path, handler)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    r = yield from client.get(request_url)
+    r = await client.get(request_url)
     assert r.status == 200
 
 
-@asyncio.coroutine
-def test_handler_metadata_persistence():
+async def test_handler_metadata_persistence():
     """
     Tests accessing metadata of a handler after registering it on the app
     router.
     """
     app = web.Application()
 
-    @asyncio.coroutine
-    def async_handler(_):
+    async def async_handler(request):
         """Doc"""
         return web.Response()
 
-    def sync_handler(_):
+    def sync_handler(request):
         """Doc"""
         return web.Response()
 
     app.router.add_get('/async', async_handler)
-    app.router.add_get('/sync', sync_handler)
+    with pytest.warns(DeprecationWarning):
+        app.router.add_get('/sync', sync_handler)
 
     for resource in app.router.resources():
         for route in resource:
             assert route.handler.__doc__ == 'Doc'
 
 
-@asyncio.coroutine
-def test_unauthorized_folder_access(tmp_dir_path, loop, test_client):
+async def test_unauthorized_folder_access(tmp_dir_path, aiohttp_client):
     """
     Tests the unauthorized access to a folder of static file server.
     Try to list a folder content of static file server when server does not
@@ -226,15 +222,14 @@ def test_unauthorized_folder_access(tmp_dir_path, loop, test_client):
 
         # Register global static route:
         app.router.add_static('/', tmp_dir_path, show_index=True)
-        client = yield from test_client(app)
+        client = await aiohttp_client(app)
 
         # Request the root of the static directory.
-        r = yield from client.get('/my_dir')
+        r = await client.get('/my_dir')
         assert r.status == 403
 
 
-@asyncio.coroutine
-def test_access_symlink_loop(tmp_dir_path, loop, test_client):
+async def test_access_symlink_loop(tmp_dir_path, aiohttp_client):
     """
     Tests the access to a looped symlink, which could not be resolved.
     """
@@ -245,15 +240,14 @@ def test_access_symlink_loop(tmp_dir_path, loop, test_client):
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
     # Request the root of the static directory.
-    r = yield from client.get('/my_symlink')
+    r = await client.get('/my_symlink')
     assert r.status == 404
 
 
-@asyncio.coroutine
-def test_access_special_resource(tmp_dir_path, loop, test_client):
+async def test_access_special_resource(tmp_dir_path, aiohttp_client):
     """
     Tests the access to a resource that is neither a file nor a directory.
     Checks that if a special resource is accessed (f.e. named pipe or UNIX
@@ -276,33 +270,30 @@ def test_access_special_resource(tmp_dir_path, loop, test_client):
 
         # Register global static route:
         app.router.add_static('/', tmp_dir_path, show_index=True)
-        client = yield from test_client(app)
+        client = await aiohttp_client(app)
 
         # Request the root of the static directory.
-        r = yield from client.get('/special')
+        r = await client.get('/special')
         assert r.status == 404
 
 
-@asyncio.coroutine
-def test_partialy_applied_handler(loop, test_client):
+async def test_partialy_applied_handler(aiohttp_client):
     app = web.Application()
 
-    @asyncio.coroutine
-    def handler(data, request):
+    async def handler(data, request):
         return web.Response(body=data)
 
-    app.router.add_route('GET', '/', functools.partial(handler, b'hello'))
-    client = yield from test_client(app)
+    with pytest.warns(DeprecationWarning):
+        app.router.add_route('GET', '/', functools.partial(handler, b'hello'))
+    client = await aiohttp_client(app)
 
-    r = yield from client.get('/')
-    data = (yield from r.read())
+    r = await client.get('/')
+    data = (await r.read())
     assert data == b'hello'
 
 
 def test_system_route():
     route = SystemRoute(web.HTTPCreated(reason='test'))
-    with pytest.raises(RuntimeError):
-        route.url()
     with pytest.raises(RuntimeError):
         route.url_for()
     assert route.name is None
@@ -312,49 +303,167 @@ def test_system_route():
     assert 'test' == route.reason
 
 
-@asyncio.coroutine
-def test_412_is_returned(test_client):
+async def test_412_is_returned(aiohttp_client):
 
     class MyRouter(abc.AbstractRouter):
 
-        @asyncio.coroutine
-        def resolve(self, request):
+        async def resolve(self, request):
             raise web.HTTPPreconditionFailed()
 
     app = web.Application(router=MyRouter())
 
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    resp = yield from client.get('/')
+    resp = await client.get('/')
 
     assert resp.status == 412
 
 
-@asyncio.coroutine
-def test_allow_head(loop, test_client):
+async def test_allow_head(aiohttp_client):
     """
     Test allow_head on routes.
     """
     app = web.Application()
 
-    def handler(_):
+    async def handler(_):
         return web.Response()
     app.router.add_get('/a', handler, name='a')
     app.router.add_get('/b', handler, allow_head=False, name='b')
-    client = yield from test_client(app)
+    client = await aiohttp_client(app)
 
-    r = yield from client.get('/a')
+    r = await client.get('/a')
     assert r.status == 200
-    yield from r.release()
+    await r.release()
 
-    r = yield from client.head('/a')
+    r = await client.head('/a')
     assert r.status == 200
-    yield from r.release()
+    await r.release()
 
-    r = yield from client.get('/b')
+    r = await client.get('/b')
     assert r.status == 200
-    yield from r.release()
+    await r.release()
 
-    r = yield from client.head('/b')
+    r = await client.head('/b')
     assert r.status == 405
-    yield from r.release()
+    await r.release()
+
+
+@pytest.mark.parametrize("path", [
+    '/a',
+    '/{a}',
+])
+def test_reuse_last_added_resource(path):
+    """
+    Test that adding a route with the same name and path of the last added
+    resource doesn't create a new resource.
+    """
+    app = web.Application()
+
+    async def handler(request):
+        return web.Response()
+
+    app.router.add_get(path, handler, name="a")
+    app.router.add_post(path, handler, name="a")
+
+    assert len(app.router.resources()) == 1
+
+
+def test_resource_raw_match():
+    app = web.Application()
+
+    async def handler(request):
+        return web.Response()
+
+    route = app.router.add_get("/a", handler, name="a")
+    assert route.resource.raw_match("/a")
+
+    route = app.router.add_get("/{b}", handler, name="b")
+    assert route.resource.raw_match("/{b}")
+
+    resource = app.router.add_static("/static", ".")
+    assert not resource.raw_match("/static")
+
+
+async def test_add_view(aiohttp_client):
+    app = web.Application()
+
+    class MyView(web.View):
+        async def get(self):
+            return web.Response()
+
+        async def post(self):
+            return web.Response()
+
+    app.router.add_view("/a", MyView)
+
+    client = await aiohttp_client(app)
+
+    r = await client.get("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.post("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.put("/a")
+    assert r.status == 405
+    await r.release()
+
+
+async def test_decorate_view(aiohttp_client):
+    routes = web.RouteTableDef()
+
+    @routes.view("/a")
+    class MyView(web.View):
+        async def get(self):
+            return web.Response()
+
+        async def post(self):
+            return web.Response()
+
+    app = web.Application()
+    app.router.add_routes(routes)
+
+    client = await aiohttp_client(app)
+
+    r = await client.get("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.post("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.put("/a")
+    assert r.status == 405
+    await r.release()
+
+
+async def test_web_view(aiohttp_client):
+    app = web.Application()
+
+    class MyView(web.View):
+        async def get(self):
+            return web.Response()
+
+        async def post(self):
+            return web.Response()
+
+    app.router.add_routes([
+        web.view("/a", MyView)
+    ])
+
+    client = await aiohttp_client(app)
+
+    r = await client.get("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.post("/a")
+    assert r.status == 200
+    await r.release()
+
+    r = await client.put("/a")
+    assert r.status == 405
+    await r.release()
